-- QuestIon by Celdor
QuestIon_VERSION = "1.6.1";

-- New in 1.6.1:
-- Key press for auto-minimize activation (Thanks to Rhidon)
-- BUG: Font scaling issue (Thanks to Parahelios! I was having a hard time tracking it down)

-- New in 1.6.0:
-- Added a German Translation
-- Disabled my ToggleDropDownMenu code
-- Fixed the UIERRORS_HOLD_TIME error

-- QuestIon (pronounced Quest - I - On) adds a customizable and configurable window for tracking quests and objectives.

-- Thanks to:
--	GDI - For my dozens of UI questions, testing, design ideas, helping to solve a MAJOR bug, and the code I ganked :)
--	Trentin - For MonkeyQuest. I got some concept ideas from your design and threads. 
--			Also helped solve some major quest function related bugs.
--	Vjeux - For the original Quest Minion
--	Kilguren - For the standalone version of Quest Minion
--	Dhargo - For BareBonesMod that helped me get started
--	Blizzard - For World of Warcraft and lots of code I found in your UI

----------------------------------

-- Binding Variables
BINDING_HEADER_QUESTIONHEADER = QUESTIONTEXT_BINDING_HEADER_QUESTIONHEADER;
BINDING_NAME_QUESTIONVISIBLE = QUESTIONTEXT_BINDING_NAME_QUESTIONVISIBLE;
BINDING_NAME_QUESTIONMINIMIZE = QUESTIONTEXT_BINDING_NAME_QUESTIONMINIMIZE;

-- Local Variables
local QuestIon_updateWhileCooling = false;
local QuestIon_eventCooldown = 0;
local QuestIon_eventCooldownTime = 1;
local QuestIon_MasterQuestPlayer = nil;
local QuestIon_MasterOptionPlayer = nil;
local QuestIon_PastFirstUNU = false;
local QuestIon_PastFirstQLU = false;
local QuestIon_PastFirstUpdate = false;
local QuestIon_inFrame = false;
local QuestIon_inTitle = false;
local QuestIon_inQuestTitle = false;
local QuestIon_enableSecondDropdownMenu = false;
local QuestIon_secondDropdownMenuLocation = "";
local QuestIon_numButtons = 40;
local Blizzard_ToggleDropDownMenu;

-- Global Variables
QuestIon_allowUpdates = true;

-- Static Dialog Windows
StaticPopupDialogs["QuestIon_FadeInAlpha"] = {
	text = QUESTIONTEXT_STATIC_FADEINALPHA,
	button1 = TEXT(ACCEPT),
	button2 = TEXT(CANCEL),
	hasEditBox = 1,
	maxLetters = 8,
	whileDead = 1,
	OnAccept = function()
		local value = tonumber(getglobal(this:GetParent():GetName().."EditBox"):GetText());
		if (type(value) ~= "number") then
			DEFAULT_CHAT_FRAME:AddMessage(QUESTIONTEXT_STATIC_FADEINALPHA_INVALID);
		elseif (value < 0 or value > 1) then
			DEFAULT_CHAT_FRAME:AddMessage(QUESTIONTEXT_STATIC_FADEINALPHA_BOUNDS .. value);
		else
			QuestIon_MasterOptionPlayer["autofade_inalpha"] = value;
		end
		getglobal(this:GetParent():GetName().."EditBox"):SetText("");
	end,
	timeout = 0,
	EditBoxOnEnterPressed = function()
		local value = tonumber(getglobal(this:GetParent():GetName().."EditBox"):GetText());
		if (type(value) ~= "number") then
			DEFAULT_CHAT_FRAME:AddMessage(QUESTIONTEXT_STATIC_FADEINALPHA_INVALID);
		elseif (value < 0 or value > 1) then
			DEFAULT_CHAT_FRAME:AddMessage(QUESTIONTEXT_STATIC_FADEINALPHA_BOUNDS .. value);
		else
			QuestIon_MasterOptionPlayer["autofade_inalpha"] = value;
		end
		getglobal(this:GetParent():GetName().."EditBox"):SetText("");
		this:GetParent():Hide();
	end
};
StaticPopupDialogs["QuestIon_FadeOutAlpha"] = {
	text = QUESTIONTEXT_STATIC_FADEOUTALPHA,
	button1 = TEXT(ACCEPT),
	button2 = TEXT(CANCEL),
	hasEditBox = 1,
	maxLetters = 8,
	whileDead = 1,
	OnAccept = function()
		local value = tonumber(getglobal(this:GetParent():GetName().."EditBox"):GetText());
		if (type(value) ~= "number") then
			DEFAULT_CHAT_FRAME:AddMessage(QUESTIONTEXT_STATIC_FADEOUTALPHA_INVALID);
		elseif (value < 0 or value > 1) then
			DEFAULT_CHAT_FRAME:AddMessage(QUESTIONTEXT_STATIC_FADEOUTALPHA_BOUNDS .. value);
		else
			QuestIon_MasterOptionPlayer["autofade_outalpha"] = value;
		end
		getglobal(this:GetParent():GetName().."EditBox"):SetText("");
	end,
	timeout = 0,
	EditBoxOnEnterPressed = function()
		local value = tonumber(getglobal(this:GetParent():GetName().."EditBox"):GetText());
		if (type(value) ~= "number") then
			DEFAULT_CHAT_FRAME:AddMessage(QUESTIONTEXT_STATIC_FADEOUTALPHA_INVALID);
		elseif (value < 0 or value > 1) then
			DEFAULT_CHAT_FRAME:AddMessage(QUESTIONTEXT_STATIC_FADEOUTALPHA_BOUNDS .. value);
		else
			QuestIon_MasterOptionPlayer["autofade_outalpha"] = value;
		end
		getglobal(this:GetParent():GetName().."EditBox"):SetText("");
		this:GetParent():Hide();
	end
};
StaticPopupDialogs["QuestIon_FadeInTime"] = {
	text = QUESTIONTEXT_STATIC_FADEINTIME,
	button1 = TEXT(ACCEPT),
	button2 = TEXT(CANCEL),
	hasEditBox = 1,
	maxLetters = 8,
	whileDead = 1,
	OnAccept = function()
		local value = tonumber(getglobal(this:GetParent():GetName().."EditBox"):GetText());
		if (type(value) ~= "number") then
			DEFAULT_CHAT_FRAME:AddMessage(QUESTIONTEXT_STATIC_FADEINTIME_INVALID);
		elseif (value < 0 or value > 10) then
			DEFAULT_CHAT_FRAME:AddMessage(QUESTIONTEXT_STATIC_FADEINTIME_BOUNDS .. value);
		else
			QuestIon_MasterOptionPlayer["autofade_intime"] = value;
		end
		getglobal(this:GetParent():GetName().."EditBox"):SetText("");
	end,
	timeout = 0,
	EditBoxOnEnterPressed = function()
		local value = tonumber(getglobal(this:GetParent():GetName().."EditBox"):GetText());
		if (type(value) ~= "number") then
			DEFAULT_CHAT_FRAME:AddMessage(QUESTIONTEXT_STATIC_FADEINTIME_INVALID);
		elseif (value < 0 or value > 10) then
			DEFAULT_CHAT_FRAME:AddMessage(QUESTIONTEXT_STATIC_FADEINTIME_BOUNDS .. value);
		else
			QuestIon_MasterOptionPlayer["autofade_intime"] = value;
		end
		getglobal(this:GetParent():GetName().."EditBox"):SetText("");
		this:GetParent():Hide();
	end
};
StaticPopupDialogs["QuestIon_FadeOutTime"] = {
	text = QUESTIONTEXT_STATIC_FADEOUTTIME,
	button1 = TEXT(ACCEPT),
	button2 = TEXT(CANCEL),
	hasEditBox = 1,
	maxLetters = 8,
	whileDead = 1,
	OnAccept = function()
		local value = tonumber(getglobal(this:GetParent():GetName().."EditBox"):GetText());
		if (type(value) ~= "number") then
			DEFAULT_CHAT_FRAME:AddMessage(QUESTIONTEXT_STATIC_FADEOUTTIME_INVALID);
		elseif (value < 0 or value > 10) then
			DEFAULT_CHAT_FRAME:AddMessage(QUESTIONTEXT_STATIC_FADEOUTTIME_BOUNDS .. value);
		else
			QuestIon_MasterOptionPlayer["autofade_outtime"] = value;
		end
		getglobal(this:GetParent():GetName().."EditBox"):SetText("");
	end,
	timeout = 0,
	EditBoxOnEnterPressed = function()
		local value = tonumber(getglobal(this:GetParent():GetName().."EditBox"):GetText());
		if (type(value) ~= "number") then
			DEFAULT_CHAT_FRAME:AddMessage(QUESTIONTEXT_STATIC_FADEOUTTIME_INVALID);
		elseif (value < 0 or value > 10) then
			DEFAULT_CHAT_FRAME:AddMessage(QUESTIONTEXT_STATIC_FADEOUTTIME_BOUNDS .. value);
		else
			QuestIon_MasterOptionPlayer["autofade_outtime"] = value;
		end
		getglobal(this:GetParent():GetName().."EditBox"):SetText("");
		this:GetParent():Hide();
	end
};
StaticPopupDialogs["QuestIon_LevelRange"] = {
	text = QUESTIONTEXT_STATIC_LEVELRANGE,
	button1 = TEXT(ACCEPT),
	button2 = TEXT(CANCEL),
	hasEditBox = 1,
	maxLetters = 8,
	whileDead = 1,
	OnAccept = function()
		local value = tonumber(getglobal(this:GetParent():GetName().."EditBox"):GetText());
		if (type(value) ~= "number") then
			DEFAULT_CHAT_FRAME:AddMessage(QUESTIONTEXT_STATIC_LEVELRANGE_INVALID);
		elseif (value < 0 or value > 10) then
			DEFAULT_CHAT_FRAME:AddMessage(QUESTIONTEXT_STATIC_LEVELRANGE_BOUNDS .. value);
		else
			QuestIon_MasterOptionPlayer["default_filterRange"] = value;
		end
		getglobal(this:GetParent():GetName().."EditBox"):SetText("");
		QuestIon_WindowUpdate();
	end,
	timeout = 0,
	EditBoxOnEnterPressed = function()
		local value = tonumber(getglobal(this:GetParent():GetName().."EditBox"):GetText());
		if (type(value) ~= "number") then
			DEFAULT_CHAT_FRAME:AddMessage(QUESTIONTEXT_STATIC_LEVELRANGE_INVALID);
		elseif (value < 0 or value > 10) then
			DEFAULT_CHAT_FRAME:AddMessage(QUESTIONTEXT_STATIC_LEVLERANGE_BOUNDS .. value);
		else
			QuestIon_MasterOptionPlayer["default_filterRange"] = value;
		end
		getglobal(this:GetParent():GetName().."EditBox"):SetText("");
		this:GetParent():Hide();
		QuestIon_WindowUpdate();
	end
};

------------------
-- Event Functions
------------------

function QuestIon_OnLoad()
-- Setup some inital things for loading

	-- Override Blizzard's ToggleDropDownMenu Function
	-- local Blizzard_ToggleDropDownMenu = ToggleDropDownMenu;
	-- ToggleDropDownMenu = ToggleDropDownMenuUpdate_ToggleDropDownMenu;

	-- Set the background color and alpha to a default
	QuestIon_Frame_Backdrop:SetBackdropColor(0,0,0,.7);

	-- Set the various slash commands
	SlashCmdList["QUESTIONCOMMAND"] = QuestIon_SlashHandler;
	SLASH_QUESTIONCOMMAND1 = "/qio";
	SLASH_QUESTIONCOMMAND2 = "/qi";

	-- Register for the needed events
	this:RegisterEvent("QUEST_LOG_UPDATE");
	this:RegisterEvent("UNIT_NAME_UPDATE");
	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("UI_INFO_MESSAGE");
	this:RegisterEvent("UNIT_QUEST_LOG_CHANGED");
	this:RegisterEvent("PARTY_MEMBER_ENABLE");
	this:RegisterEvent("PARTY_MEMBER_DISABLE");
	this:RegisterEvent("MINIMAP_ZONE_CHANGED");
	this:RegisterEvent("ZONE_CHANGED");
	this:RegisterEvent("ZONE_CHANGED_NEW_AREA");
	this:RegisterEvent("CHAT_MSG_SYSTEM");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");

	-- Check for the Master Quest Info (all quest settings), set to an empty array if needed
	if (QuestIon_MasterQuestInfo==nil) then
		QuestIon_MasterQuestInfo = {};
	end

	-- Check for the Master Option Info (all QuestIon options), set to an empty array if needed
	if (QuestIon_MasterOptionInfo==nil) then
		QuestIon_MasterOptionInfo = {};
	end

	ShowUIPanel(QuestIon_Frame);
end

function QuestIon_OnEvent(event, ui_message)
-- Watch for the events and deal with them
	-- Is the event a QUEST_LOG_UPDATE and are updates currently disallowed?
	if (event == "QUEST_LOG_UPDATE" and QuestIon_allowUpdates == false) then
		-- If so, recored that a QUEST_LOG_UPDATE occurred while updates were disallowed
		QuestIon_updateWhileCooling = true;

		-- Leave the function
		return;
	end

	if (event == "VARIABLES_LOADED") then
		if (QuestIon_MasterOptionInfo["font_title_size"] == nil) then
			QuestIon_MasterOptionInfo["font_title_size"] = 12;
		end
		if (QuestIon_MasterOptionInfo["font_objective_size"] == nil) then
			QuestIon_MasterOptionInfo["font_objective_size"] = 10;
		end

		-- Due to how fonts work, I have to set them up here and they really have to be a global setting
		for index = 1, QuestIon_numButtons, 1 do
			local QuestIon_QuestText = getglobal("QuestIon_QuestTitle_" .. index .. "_Text");
			QuestIon_QuestText:SetTextHeight(QuestIon_MasterOptionInfo["font_title_size"]);
	
			local QuestIon_QuestObjectives = getglobal("QuestIon_QuestTitle_" .. index .. "_Objectives");
			QuestIon_QuestObjectives:SetTextHeight(QuestIon_MasterOptionInfo["font_objective_size"]);
		end
		--Parahelios Fix
		QuestIon_Frame:SetScale(2);
		QuestIon_Frame:SetScale(UIParent:GetScale());
		return;
	end

	if (event == "PLAYER_ENTERING_WORLD") then
		
		-- If so, is the player really loaded?

		-- Grab the current player's name
		local playerName = UnitName("player");
		-- Is the player's name unknown?
		if (playerName ~= UNKNOWNOBJECT) then
			-- If not, get their settings loaded

			-- Set these 2 variables up for the current Server/Character
			QuestIon_MasterQuestPlayer = QuestIon_Quest_GetPlayer();
			QuestIon_MasterOptionPlayer = QuestIon_Option_GetPlayer();

			-- Populate default QuestIon Options and apply them:

			-- Set the default options for new quests
			if (QuestIon_MasterOptionPlayer["default_isShown"] == nil) then
				QuestIon_MasterOptionPlayer["default_isShown"] = true;
			end
			if (QuestIon_MasterOptionPlayer["default_isShownComplete"] == nil) then
				QuestIon_MasterOptionPlayer["default_isShownComplete"] = true;
			end
			if (QuestIon_MasterOptionPlayer["default_showObjectives"] == nil) then
				QuestIon_MasterOptionPlayer["default_showObjectives"] = true;
			end
			if (QuestIon_MasterOptionPlayer["default_showCompleteObjectives"] == nil) then
				QuestIon_MasterOptionPlayer["default_showCompleteObjectives"] = false;
			end
			if (QuestIon_MasterOptionPlayer["default_locationInTitle"] == nil) then
				QuestIon_MasterOptionPlayer["default_locationInTitle"] = false;
			end
			if (QuestIon_MasterOptionPlayer["default_locationInObjectives"] == nil) then
				QuestIon_MasterOptionPlayer["default_locationInObjectives"] = false;
			end
			if (QuestIon_MasterOptionPlayer["default_showOnObjectivePickup"] == nil) then
				QuestIon_MasterOptionPlayer["default_showOnObjectivePickup"] = false;
			end
			if (QuestIon_MasterOptionPlayer["default_showPartyInfo"] == nil) then
				QuestIon_MasterOptionPlayer["default_showPartyInfo"] = false;
			end
			if (QuestIon_MasterOptionPlayer["default_showNotOnQuest"] == nil) then
				QuestIon_MasterOptionPlayer["default_showNotOnQuest"] = true;
			end


			-- Set the default colors
			if (QuestIon_MasterOptionPlayer["color_obj_start_red"] == nil) then
				QuestIon_MasterOptionPlayer["color_obj_start_red"] = 153;
			end
			if (QuestIon_MasterOptionPlayer["color_obj_start_green"] == nil) then
				QuestIon_MasterOptionPlayer["color_obj_start_green"] = 153;
			end
			if (QuestIon_MasterOptionPlayer["color_obj_start_blue"] == nil) then
				QuestIon_MasterOptionPlayer["color_obj_start_blue"] = 153;
			end
			if (QuestIon_MasterOptionPlayer["color_obj_end_red"] == nil) then
				QuestIon_MasterOptionPlayer["color_obj_end_red"] = 0;
			end
			if (QuestIon_MasterOptionPlayer["color_obj_end_green"] == nil) then
				QuestIon_MasterOptionPlayer["color_obj_end_green"] = 128;
			end
			if (QuestIon_MasterOptionPlayer["color_obj_end_blue"] == nil) then
				QuestIon_MasterOptionPlayer["color_obj_end_blue"] = 255;
			end
			if (QuestIon_MasterOptionPlayer["red_frame_backdrop"] == nil) then
				QuestIon_MasterOptionPlayer["red_frame_backdrop"] = 0;
			end
			if (QuestIon_MasterOptionPlayer["green_frame_backdrop"] == nil) then
				QuestIon_MasterOptionPlayer["green_frame_backdrop"] = 0;
			end
			if (QuestIon_MasterOptionPlayer["blue_frame_backdrop"] == nil) then
				QuestIon_MasterOptionPlayer["blue_frame_backdrop"] = 0;
			end
			if (QuestIon_MasterOptionPlayer["alpha_frame_backdrop"] == nil) then
				QuestIon_MasterOptionPlayer["alpha_frame_backdrop"] = 0.7;
			end
			QuestIon_Frame_Backdrop:SetBackdropColor(QuestIon_MasterOptionPlayer["red_frame_backdrop"],QuestIon_MasterOptionPlayer["green_frame_backdrop"],QuestIon_MasterOptionPlayer["blue_frame_backdrop"],QuestIon_MasterOptionPlayer["alpha_frame_backdrop"]);

			if (QuestIon_MasterOptionPlayer["red_frame_border"] == nil) then
				QuestIon_MasterOptionPlayer["red_frame_border"] = 1;
			end
			if (QuestIon_MasterOptionPlayer["green_frame_border"] == nil) then
				QuestIon_MasterOptionPlayer["green_frame_border"] = 1;
			end
			if (QuestIon_MasterOptionPlayer["blue_frame_border"] == nil) then
				QuestIon_MasterOptionPlayer["blue_frame_border"] = 1;
			end
			if (QuestIon_MasterOptionPlayer["alpha_frame_border"] == nil) then
				QuestIon_MasterOptionPlayer["alpha_frame_border"] = 1;
			end
			QuestIon_Frame_Backdrop:SetBackdropBorderColor(QuestIon_MasterOptionPlayer["red_frame_border"],QuestIon_MasterOptionPlayer["green_frame_border"],QuestIon_MasterOptionPlayer["blue_frame_border"],QuestIon_MasterOptionPlayer["alpha_frame_border"]);

			-- DEBUG:
			if (QuestIon_MasterOptionPlayer["color_complete_quest_red"] == nil) then
				QuestIon_MasterOptionPlayer["color_complete_quest_red"] = 128;
			end
			if (QuestIon_MasterOptionPlayer["color_complete_quest_green"] == nil) then
				QuestIon_MasterOptionPlayer["color_complete_quest_green"] = 96;
			end
			if (QuestIon_MasterOptionPlayer["color_complete_quest_blue"] == nil) then
				QuestIon_MasterOptionPlayer["color_complete_quest_blue"] = 192;
			end
			if (QuestIon_MasterOptionPlayer["show_completeTag"] == nil) then
				QuestIon_MasterOptionPlayer["show_completeTag"] = true;
			end
			-- :DEBUG

			-- Set if the QuestIon window should be minimized
			if (QuestIon_MasterOptionPlayer["frame_minimized"] == nil) then
				QuestIon_MasterOptionPlayer["frame_minimized"] = false;
			end

			-- Set if the QuestIon window is hidden
			if (QuestIon_MasterOptionPlayer["frame_hidden"] == nil) then
				QuestIon_MasterOptionPlayer["frame_hidden"] = false;
			end

			-- Set which corner QuestIon grows from
			if (QuestIon_MasterOptionPlayer["corner_mode"] == nil) then
				QuestIon_MasterOptionPlayer["corner_mode"] = 4;
			end

			-- Set if the Minimize and Close buttons are hidden
			if (QuestIon_MasterOptionPlayer["hide_buttons"] == nil) then
				QuestIon_MasterOptionPlayer["hide_buttons"] = false;
			end
			if (QuestIon_MasterOptionPlayer["hide_buttons"]) then
				QuestIon_CloseButton:Hide();
				QuestIon_MinimizeButton:Hide();
			else
				QuestIon_CloseButton:Show();
				QuestIon_MinimizeButton:Show();
			end

			-- Set if Auto-Fade is enabled
			if (QuestIon_MasterOptionPlayer["autofade_enabled"] == nil) then
				QuestIon_MasterOptionPlayer["autofade_enabled"] = true;
			end
			if (QuestIon_MasterOptionPlayer["autofade_inalpha"] == nil) then
				QuestIon_MasterOptionPlayer["autofade_inalpha"] = 1.0;
			end
			if (QuestIon_MasterOptionPlayer["autofade_intime"] == nil) then
				QuestIon_MasterOptionPlayer["autofade_intime"] = 0.25;
			end
			if (QuestIon_MasterOptionPlayer["autofade_outalpha"] == nil) then
				QuestIon_MasterOptionPlayer["autofade_outalpha"] = 0.25;
			end
			if (QuestIon_MasterOptionPlayer["autofade_outtime"] == nil) then
				QuestIon_MasterOptionPlayer["autofade_outtime"] = 2.0;
			end

			-- Set if Auto-Minimized is enabled
			if (QuestIon_MasterOptionPlayer["autominimize_enabled"] == nil) then
				QuestIon_MasterOptionPlayer["autominimize_enabled"] = false;
			end

			-- Set if a hover restore key is set
			if (QuestIon_MasterOptionPlayer["hover_restore_key"] == nil) then
				QuestIon_MasterOptionPlayer["hover_restore_key"] = false;
			end

			-- Set if Quest Tooltips appear
			if (QuestIon_MasterOptionPlayer["frame_quest_tooltips"] == nil) then
				QuestIon_MasterOptionPlayer["frame_quest_tooltips"] = true;
			end

			-- Set Quest Fading
			if (QuestIon_MasterOptionPlayer["quest_fading_enabled"] == nil) then
				QuestIon_MasterOptionPlayer["quest_fading_enabled"] = true;
			end
			if (QUEST_FADING_ENABLE) then
				QUEST_FADING_ENABLE = QuestIon_MasterOptionPlayer["quest_fading_enabled"];
			end

			if (QuestIon_MasterOptionPlayer["additional_font_sizes"] == nil) then
				QuestIon_MasterOptionPlayer["additional_font_sizes"] = false;
			end
			if (QuestIon_MasterOptionPlayer["additional_font_sizes"]) then
				CHAT_FONT_HEIGHTS = {
					[1] = 8,
					[2] = 10,
					[3] = 11,
					[4] = 12,
					[5] = 13,
					[6] = 14,
					[7] = 15,
					[8] = 16,
					[9] = 18,
					[10] = 20
				};
			end

			if (QuestIon_MasterOptionPlayer["default_sortMode"] == nil) then
				QuestIon_MasterOptionPlayer["default_sortMode"] = "normal";
			end
			if (QuestIon_MasterOptionPlayer["default_filterByLevel"] == nil) then
				QuestIon_MasterOptionPlayer["default_filterByLevel"] = false;
			end
			if (QuestIon_MasterOptionPlayer["default_filterRange"] == nil) then
				QuestIon_MasterOptionPlayer["default_filterRange"] = 2;
			end
			if (QuestIon_MasterOptionPlayer["default_filterByLocation"] == nil) then
				QuestIon_MasterOptionPlayer["default_filterByLocation"] = false;
			end
			if (QuestIon_MasterOptionPlayer["windowLocked"] == nil) then
				QuestIon_MasterOptionPlayer["windowLocked"] = false;
			end
			if (QuestIon_MasterOptionPlayer["playSoundQuestComplete"] == nil) then
				QuestIon_MasterOptionPlayer["playSoundQuestComplete"] = false;
			end
			if (QuestIon_MasterOptionPlayer["showLocationHeaders"] == nil) then
				QuestIon_MasterOptionPlayer["showLocationHeaders"] = true;
			end
			if (QuestIon_MasterOptionPlayer["unminimizeOnPickup"] == nil) then
				QuestIon_MasterOptionPlayer["unminimizeOnPickup"] = false;
			end
			if (QuestIon_MasterOptionPlayer["hideQuestDescriptions"] == nil) then
				QuestIon_MasterOptionPlayer["hideQuestDescriptions"] = false;
			end

			-- Record that we've done our first successful Unit Name Update
			QuestIon_PastFirstUNU = true;
			return;
		end
	end

	-- Make sure we have our Quest and Option info (Not sure if this is still needed)
	if (QuestIon_MasterQuestPlayer == nil or QuestIon_MasterOptionPlayer == nil) then
		return;
	end

	-- Is the event a QUEST_LOG_UPDATE and are updates allowed?
	if (event == "QUEST_LOG_UPDATE" and QuestIon_allowUpdates == true) then
		-- If so, Update
		QuestIon_WindowUpdate();

	elseif (event == "UI_INFO_MESSAGE") then
		QuestIon_InfoMessageUpdate(ui_message);

	elseif (event == "UNIT_QUEST_LOG_CHANGED" or event == "PARTY_MEMBER_ENABLE" or event == "PARTY_MEMBER_DISABLE") then
		if (QuestIon_allowUpdates == false) then
			-- If so, recored that a QUEST_LOG_UPDATE occurred while updates were disallowed
			QuestIon_updateWhileCooling = true;

			-- Leave the function
			return;
		end
		QuestIon_WindowUpdate();

	-- DEBUG:
	elseif (QuestIon_MasterOptionPlayer["default_filterByLocation"] and (event == "MINIMAP_ZONE_CHANGED" or event == "ZONE_CHANGED" or event == "ZONE_CHANGED_NEW_AREA")) then
		if (QuestIon_allowUpdates == false) then
			-- If so, recored that a QUEST_LOG_UPDATE occurred while updates were disallowed
			QuestIon_updateWhileCooling = true;

			-- Leave the function
			return;
		end
		QuestIon_WindowUpdate();
	-- :DEBUG
	
	elseif (event == "CHAT_MSG_SYSTEM") then
		local searchStr = string.gsub(ERR_QUEST_ACCEPTED_S, "%%s", "");
		local _, _, questName = string.find(arg1, searchStr .. "(.+)");
		if (questName ~= nil and QuestIon_MasterQuestPlayer[questName] ~= nil) then
			-- Set defaults if the quest is new
			QuestIon_MasterQuestPlayer[questName]["isShown"] = QuestIon_MasterOptionPlayer["default_isShown"];
			QuestIon_MasterQuestPlayer[questName]["isShownComplete"] = QuestIon_MasterOptionPlayer["default_isShownComplete"];
			QuestIon_MasterQuestPlayer[questName]["showObjectives"] = QuestIon_MasterOptionPlayer["default_showObjectives"];
			QuestIon_MasterQuestPlayer[questName]["showCompleteObjectives"] = QuestIon_MasterOptionPlayer["default_showCompleteObjectives"];
			QuestIon_MasterQuestPlayer[questName]["locationInTitle"] = QuestIon_MasterOptionPlayer["default_locationInTitle"];
			QuestIon_MasterQuestPlayer[questName]["locationInObjectives"] = QuestIon_MasterOptionPlayer["default_locationInObjectives"];
			QuestIon_MasterQuestPlayer[questName]["showOnObjectivePickup"] = QuestIon_MasterOptionPlayer["default_showOnObjectivePickup"];
			QuestIon_MasterQuestPlayer[questName]["showPartyInfo"] = QuestIon_MasterOptionPlayer["default_showPartyInfo"];
			QuestIon_MasterQuestPlayer[questName]["showNotOnQuest"] = QuestIon_MasterOptionPlayer["default_showNotOnQuest"];
		end
	end
end

function QuestIon_OnUpdate(time)
	if (not QuestIon_allowUpdates) then
		QuestIon_eventCooldown = QuestIon_eventCooldown + time;
		if (QuestIon_eventCooldown > QuestIon_eventCooldownTime) then
			if (QuestIon_updateWhileCooling) then
				QuestIon_WindowUpdate();
				QuestIon_updateWhileCooling = false;
			end
			QuestIon_eventCooldown = 0;
			QuestIon_allowUpdates = true;
		end
	end
end

function QuestIon_OnEnter()
	-- Auto-Minimize
	if (QuestIon_MasterOptionPlayer["autominimize_enabled"] and QuestIon_MasterOptionPlayer["frame_minimized"]) then
		if (QuestIon_MasterOptionPlayer["hover_restore_key"] == false) then
			QuestIon_SetRestore();
		elseif (QuestIon_MasterOptionPlayer["hover_restore_key"] == "alt" and IsAltKeyDown()) then
			QuestIon_SetRestore();
		elseif (QuestIon_MasterOptionPlayer["hover_restore_key"] == "ctrl" and IsControlKeyDown()) then
			QuestIon_SetRestore();
		elseif (QuestIon_MasterOptionPlayer["hover_restore_key"] == "shift" and IsShiftKeyDown()) then
			QuestIon_SetRestore();
		end
	end

	-- Auto-Fade
	if (QuestIon_MasterOptionPlayer["autofade_enabled"]) then
		if (QuestIon_Frame:GetAlpha() ~= QuestIon_MasterOptionPlayer["autofade_inalpha"]) then
			UIFrameFadeRemoveFrame(QuestIon_Frame);
			UIFrameFadeIn(QuestIon_Frame, QuestIon_MasterOptionPlayer["autofade_intime"], QuestIon_Frame:GetAlpha(), QuestIon_MasterOptionPlayer["autofade_inalpha"]);
		end
	end
end

function QuestIon_OnLeave()
	if (MouseIsOver(QuestIon_Frame) == nil) then
		-- Auto-Minimize 
		if (QuestIon_MasterOptionPlayer["autominimize_enabled"]) then
			QuestIon_SetMinimized();
		end

		-- Auto-Fade
		if (QuestIon_MasterOptionPlayer["autofade_enabled"]) then
			UIFrameFadeRemoveFrame(QuestIon_Frame);
			UIFrameFadeOut(QuestIon_Frame, QuestIon_MasterOptionPlayer["autofade_outtime"], QuestIon_Frame:GetAlpha(), QuestIon_MasterOptionPlayer["autofade_outalpha"]);
		end
	end
end

function QuestIon_QuestTitle_OnEnter()
	if (QuestIon_MasterOptionPlayer["frame_quest_tooltips"]) then
		local questSelected = GetQuestLogSelection();

		local questToDisplay = this:GetID();

		SelectQuestLogEntry(questToDisplay);

		local currentQuestLocation = "";
		for entryIndex = 1, questToDisplay, 1 do
			local questLogTitleText, level, questTag, isHeader, isCollapsed, isComplete = GetQuestLogTitle(entryIndex);
			if (isHeader) then
				currentQuestLocation = questLogTitleText;
			end
		end

		-- Gather some information on our current Quest Log Entry
		local questLogTitleText, level, questTag, isHeader, isCollapsed, isComplete = GetQuestLogTitle(questToDisplay);

		if (isHeader) then
			return;
		end

		-- Is the quest complete?
		if ( isComplete ) then
			-- If so, set the questTag
			questTag = COMPLETE;
		end

		-- Is the quest failed?
		if ( IsCurrentQuestFailed() ) then
			-- If so, set the questTag
			questTag = FAILED;
		end

		-- Get the Quest Title color using Blizzard's function from QuestLogFrame.lua
		local levelColor = GetDifficultyColor(level);

		-- Convert the levelColor to the |caarrggbb format
		local formattedLevelColor = string.format("|cff%02x%02x%02x", levelColor.r * 255, levelColor.g * 255, levelColor.b * 255);

		local questDescription, questObjectives = GetQuestLogQuestText();
		SelectQuestLogEntry(questSelected);

		GameTooltip_SetDefaultAnchor(GameTooltip, this);
		if (questTag ~= nil) then
			GameTooltip:SetText(formattedLevelColor .. questLogTitleText .. "|r |cffffffff(" .. questTag .. ")|r");
		else
			GameTooltip:SetText(formattedLevelColor .. questLogTitleText .. "|r");
		end
		GameTooltip:AddLine("|cffffffff" .. currentQuestLocation .. "|r\n", NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1);
		GameTooltip:AddLine("|cffcccccc -" .. questObjectives .. "|r", NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 1);
		GameTooltip:Show();
	end

	-- Auto-Minimize
	if (QuestIon_MasterOptionPlayer["autominimize_enabled"] and QuestIon_MasterOptionPlayer["frame_minimized"]) then
		if (QuestIon_MasterOptionPlayer["hover_restore_key"] == false) then
			QuestIon_SetRestore();
		elseif (QuestIon_MasterOptionPlayer["hover_restore_key"] == "alt" and IsAltKeyDown()) then
			QuestIon_SetRestore();
		elseif (QuestIon_MasterOptionPlayer["hover_restore_key"] == "ctrl" and IsControlKeyDown()) then
			QuestIon_SetRestore();
		elseif (QuestIon_MasterOptionPlayer["hover_restore_key"] == "shift" and IsShiftKeyDown()) then
			QuestIon_SetRestore();
		end
	end

	-- Auto-Fade
	if (QuestIon_MasterOptionPlayer["autofade_enabled"]) then
		if (QuestIon_Frame:GetAlpha() ~= QuestIon_MasterOptionPlayer["autofade_inalpha"]) then
			UIFrameFadeRemoveFrame(QuestIon_Frame);
			UIFrameFadeIn(QuestIon_Frame, QuestIon_MasterOptionPlayer["autofade_intime"], QuestIon_Frame:GetAlpha(), QuestIon_MasterOptionPlayer["autofade_inalpha"]);
		end
	end
end

function QuestIon_QuestTitle_OnLeave()
	if (MouseIsOver(QuestIon_Frame) == nil) then
		-- Auto-Minimize
		if (QuestIon_MasterOptionPlayer["autominimize_enabled"]) then
			QuestIon_SetMinimized();
		end

		-- Auto-Fade
		if (QuestIon_MasterOptionPlayer["autofade_enabled"]) then
			UIFrameFadeRemoveFrame(QuestIon_Frame);
			UIFrameFadeOut(QuestIon_Frame, QuestIon_MasterOptionPlayer["autofade_outtime"], QuestIon_Frame:GetAlpha(), QuestIon_MasterOptionPlayer["autofade_outalpha"]);
		end
	end
	if (QuestIon_MasterOptionPlayer["frame_quest_tooltips"]) then
		GameTooltip:FadeOut();
	end
end

function QuestIon_SlashHandler(msg)
-- We deal with the slash commands here

	-- Do we have a message?
	if (msg) then
		-- If so, process it

		-- Set the message to be lower case
		msg = string.lower(msg);

		-- Is it the show command?
		if (string.find(msg, "show")) then
			-- If so, update the Master Options
			QuestIon_MasterOptionPlayer["frame_hidden"] = false;

			-- Show the QuestIon window
			ShowUIPanel(QuestIon_Frame);

			-- Update the Window
			QuestIon_WindowUpdate();

		-- Is it the hide command?
		elseif (string.find(msg, "hide")) then
			-- If so, update the Master Options
			QuestIon_MasterOptionPlayer["frame_hidden"] = true;

			-- Hide the QuestIon window
			HideUIPanel(QuestIon_Frame);

		-- Is it the reset position command?
		elseif (string.find(msg, "resetpos")) then

			-- If so, update the Master Options
			QuestIon_MasterOptionPlayer["corner_mode"] = 4;

			-- Reset the QuestIon window position and alpha
			QuestIon_Frame:ClearAllPoints();
			QuestIon_Frame:SetPoint("BOTTOMRIGHT", "UIParent", "BOTTOMLEFT", 600, 90);
			QuestIon_Frame:SetAlpha(1);
			
		-- Is it a blank/unknown command?
		else
			-- Print the slash command help
			QuestIon_ChatPrint("QuestIon Usage: /qi [show/hide/resetpos]");
		end
	end
end

function QuestIon_QuestTitle_OnClick(button)
-- Handles the mouse clicks on the Quest Titles
-- Left click: Open the Quest Log to the selected quest
-- Shift Left click: Copy the Quest Title to the chat edit box
-- Right click: Popup menu for per-quest options
-- Shift Right click: Copy the objectives to the chat edit box
-- Ctrl Left click: Shares the Quest
-- Ctrl Right click: Abandons the Quest

	-- Which mouse button was used?
	if (button == "LeftButton") then
		-- If it was the LeftButton, what it a shift click?
		if ( IsShiftKeyDown() and ChatFrameEditBox:IsVisible() ) then
			-- If so, copy the quest title to the chat edit box
			local QuestIon_TitleClicked = getglobal(this:GetName().."_PlainTitle");
			local questLogTitleText, level, questTag, isHeader, isCollapsed, isComplete = GetQuestLogTitle(this:GetID());

			-- Is the quest complete?
			if (QuestIon_MasterQuestPlayer[questLogTitleText]["isComplete"] ) then
				-- If so, set the questTag
				questTag = COMPLETE;
			end

			-- Is the quest failed?
			if ( IsCurrentQuestFailed() ) then
				-- If so, set the questTag
				questTag = FAILED;
			end

			if (level ~= 0) then
				if (questTag ~= nil) then
					ChatFrameEditBox:Insert("[" .. level .. "] " .. QuestIon_TitleClicked:GetText() .. " (" .. questTag .. ") ");
				else
					ChatFrameEditBox:Insert("[" .. level .. "] " .. QuestIon_TitleClicked:GetText() .. " ");
				end
			else
				ChatFrameEditBox:Insert(QuestIon_TitleClicked:GetText() .. " ");
			end
			-- Leave the function
			return;
		-- DEBUG:
		elseif ( IsShiftKeyDown() ) then

				-- Shift-click toggles quest-watch on this quest.
				if ( IsQuestWatched(this:GetID()) ) then
					RemoveQuestWatch(this:GetID());
					QuestWatch_Update();
				else
					-- Set error if no objectives
					if ( GetNumQuestLeaderBoards(this:GetID()) == 0 ) then
						UIErrorsFrame:AddMessage(QUEST_WATCH_NO_OBJECTIVES, 1.0, 0.1, 0.1, 1.0, 3);
						return;
					end
					-- Set an error message if trying to show too many quests
					if ( GetNumQuestWatches() >= MAX_WATCHABLE_QUESTS ) then
						UIErrorsFrame:AddMessage(format(QUEST_WATCH_TOO_MANY, MAX_WATCHABLE_QUESTS), 1.0, 0.1, 0.1, 1.0, 3);
						return;
					end
					AddQuestWatch(this:GetID());
					QuestWatch_Update();
				end
				return;
		-- :DEBUG		
		end

		-- If it was the LeftButton, show that quest in the Quest Log

		-- Get the button that was clicked
		local QuestIon_TitleClicked = getglobal(this:GetName());

		local questLogTitleText, level, questTag, isHeader, isCollapsed, isComplete = GetQuestLogTitle(this:GetID());

		if ( IsControlKeyDown() and not isHeader) then
			--Share Quest
			local questSelected = GetQuestLogSelection();

			SelectQuestLogEntry(this:GetID());

			if (GetQuestLogPushable() and GetNumPartyMembers() > 0) then
				QuestLogPushQuest();
			end

			SelectQuestLogEntry(questSelected);
			return;
		end

		if (QuestIon_MasterQuestPlayer[questLogTitleText]["headerIsMinimized"] ~= nil) then
			--A click on a location header
			if (QuestIon_MasterQuestPlayer[questLogTitleText]["headerIsMinimized"]) then
				QuestIon_MasterQuestPlayer[questLogTitleText]["headerIsMinimized"] = false;
			else
				QuestIon_MasterQuestPlayer[questLogTitleText]["headerIsMinimized"] = true;
			end
			QuestIon_WindowUpdate();
		else
			-- You have to bounce the Quest Log for the selected quest to update
			HideUIPanel(QuestLogFrame);

			-- Set the Quest Log entry to show based on the button's ID (Set in QuestIon_WindowUpdate)
			QuestLog_SetSelection(QuestIon_TitleClicked:GetID());

			-- Scroll down to the right quest
			if (QuestIon_TitleClicked:GetID() > QUESTS_DISPLAYED) then
				FauxScrollFrame_SetOffset(QuestLogListScrollFrame, QuestIon_TitleClicked:GetID() - QUESTS_DISPLAYED);
			else
			 	FauxScrollFrame_SetOffset(QuestLogListScrollFrame, 0);
			end

			-- Keeps the color highlight in sync with the current quest
			QuestLog_Update();

			-- Show the Quest Log with the newly selected entry
			ShowUIPanel(QuestLogFrame);
		end

	elseif (button == "RightButton") then
		local questLogTitleText, level, questTag, isHeader, isCollapsed, isComplete = GetQuestLogTitle(this:GetID());

		if ( IsControlKeyDown() and not isHeader) then
			-- Abandon Quest
			local questSelected = GetQuestLogSelection();

			SelectQuestLogEntry(this:GetID());

			SetAbandonQuest();
			StaticPopup_Show("ABANDON_QUEST", GetAbandonQuestName());

			SelectQuestLogEntry(questSelected);
			return;
		end

		-- If it was the RightButton, open the per-quest option menu
		if ( IsShiftKeyDown() and ChatFrameEditBox:IsVisible() and not isHeader) then
			-- If so, copy the quest title to the chat edit box

			-- Record the currently selected quest in the Quest Log
			local questSelected = GetQuestLogSelection();

			-- Track if the Quest Log was visible
			local wasQuestLogVisible = false;

			-- Check if the Quest Log was visible
			if ( QuestLogFrame:IsVisible() ) then
				-- Hide the Quest Log while we update
				HideUIPanel(QuestLogFrame);

				-- Record that the Quest Log was visible
				wasQuestLogVisible = true;
			end
			SelectQuestLogEntry(this:GetID());
			local numObjectives = GetNumQuestLeaderBoards();
			local QuestIon_QuestObjectivesText = "";
			if (numObjectives ~= 0) then
				for objectiveIndex = 1, numObjectives, 1 do
					local text, quest_type, finished = GetQuestLogLeaderBoard(objectiveIndex);
					if ( not text or strlen(text) == 0 ) then
						text = quest_type;
					end
					QuestIon_QuestObjectivesText = QuestIon_QuestObjectivesText .. "<" .. text .. "> ";
				end
			else
				local questDescription, questObjectives = GetQuestLogQuestText();
				QuestIon_QuestObjectivesText = "<" .. questObjectives .. "> ";
			end
			ChatFrameEditBox:Insert(QuestIon_QuestObjectivesText);
			SelectQuestLogEntry(questSelected);

			if (wasQuestLogVisible) then
				ShowUIPanel(QuestLogFrame);
			end
		else
			local cursorX, cursorY = GetCursorPosition();
			ToggleDropDownMenu(1, this:GetName(), getglobal(this:GetName().."_DropDown"), "UIParent", cursorX, cursorY);
		end
	end
end

function QuestIon_MouseDown(button)
	if (button == "LeftButton") then
		if (not QuestIon_MasterOptionPlayer["windowLocked"]) then
			getglobal("QuestIon_Frame"):StartMoving();
		end
	else
		local cursorX, cursorY = GetCursorPosition();
		ToggleDropDownMenu(1, this:GetName(), getglobal(this:GetName().."_DropDown"), "UIParent", cursorX, cursorY);
	end
end

function QuestIon_MouseUp(button)
	QuestIon_Frame:StopMovingOrSizing();
	local new_left = QuestIon_Frame:GetLeft();
	local new_top = QuestIon_Frame:GetTop();
	QuestIon_Frame:ClearAllPoints();
	if (QuestIon_MasterOptionPlayer["corner_mode"] == 1) then
		QuestIon_Frame:SetPoint("TOPLEFT", "UIParent", "BOTTOMLEFT", new_left, new_top);
	elseif (QuestIon_MasterOptionPlayer["corner_mode"] == 2) then
		QuestIon_Frame:SetPoint("TOPRIGHT", "UIParent", "BOTTOMLEFT", new_left + QuestIon_Frame:GetWidth(), new_top);
	elseif (QuestIon_MasterOptionPlayer["corner_mode"] == 3) then
		QuestIon_Frame:SetPoint("BOTTOMLEFT", "UIParent", "BOTTOMLEFT", new_left, new_top - QuestIon_Frame:GetHeight());
	else
		QuestIon_Frame:SetPoint("BOTTOMRIGHT", "UIParent", "BOTTOMLEFT", new_left + QuestIon_Frame:GetWidth(), new_top - QuestIon_Frame:GetHeight());
	end
end

-----------------
-- Main Functions
-----------------

function QuestIon_WindowUpdate()
-- This is the driving function. It rebuilds the entire QuestIon_Frame

	if (not QuestIon_PastFirstUNU) then
		return;
	end

	if (not QuestIon_PastFirstQLU) then
		QuestIon_PastFirstQLU = true;

		-- This makes sure a second WindowUpdate occurs to fix a bug when you login for the first time
		QuestIon_allowUpdates = false;
		QuestIon_updateWhileCooling = true;
		return;
	end
	-- Get how many entries (quests + headers) and quests are in the Quest Log
	local numEntries, numQuests = GetNumQuestLogEntries()

	QuestIon_Frame_Title:SetText("QuestIon  [" .. numQuests .. "/20]");

	-- Colors used for the objectives
	obj_start_red = QuestIon_MasterOptionPlayer["color_obj_start_red"];
	obj_start_green = QuestIon_MasterOptionPlayer["color_obj_start_green"];
	obj_start_blue = QuestIon_MasterOptionPlayer["color_obj_start_blue"];
	obj_end_red = QuestIon_MasterOptionPlayer["color_obj_end_red"];
	obj_end_green = QuestIon_MasterOptionPlayer["color_obj_end_green"];
	obj_end_blue = QuestIon_MasterOptionPlayer["color_obj_end_blue"];

	-- Track if the Quest Log was visible
	local wasQuestLogVisible = false;

	-- Check if the Quest Log was visible
	if ( QuestLogFrame:IsVisible() ) then
		-- Hide the Quest Log while we update
		HideUIPanel(QuestLogFrame);

		-- Record that the Quest Log was visible
		wasQuestLogVisible = true;
	end

	-- Record the currently selected quest in the Quest Log
	local questSelected = GetQuestLogSelection();

	-- Track the last header text
	local currentQuestLocation = "";

	-- Track the needed QuestIon_Frame height
	local QuestIon_totalHeight = 0;

	-- Track the maximum width of a Quest Title + Quest Tag or an Objective
	-- We set this to 145 so the window doesn't get too small
	local QuestIon_maxWidth = 145;
	if (QuestIon_MasterOptionPlayer["hide_buttons"]) then
		QuestIon_maxWidth = 105;
	end

	-- Setup array to store quest and headers in
	local QuestIon_QuestHeaderArray = {};
	local QuestIon_QuestHeaderIndex = 0;

	local QuestIon_QuestInLocation = false;
	local QuestIon_tempLocationInfo = {};

	local sortMode = QuestIon_MasterOptionPlayer["default_sortMode"];
	local filterByLevel = QuestIon_MasterOptionPlayer["default_filterByLevel"];
	local filterRange = QuestIon_MasterOptionPlayer["default_filterRange"];
	local filterByLocation = QuestIon_MasterOptionPlayer["default_filterByLocation"];

	-- Go through the entire list in the Quest Log
	-- Get all the info we need to sort
	-- Setup defaults for new quests
	for entryIndex = 1, numEntries, 1 do
		-- Gather some information on our current Quest Log Entry
		local questLogTitleText, level, questTag, isHeader, isCollapsed, isComplete = GetQuestLogTitle(entryIndex);

		if (isHeader) then
			-- Set defaults if the header is new
			if (QuestIon_MasterQuestPlayer[questLogTitleText] == nil) then
				QuestIon_MasterQuestPlayer[questLogTitleText] = {};
			end
			if (QuestIon_MasterQuestPlayer[questLogTitleText]["headerIsMinimized"] == nil) then
				QuestIon_MasterQuestPlayer[questLogTitleText]["headerIsMinimized"] = false;
			end
			if (QuestIon_MasterQuestPlayer[questLogTitleText]["headerIsChecked"] == nil) then
				QuestIon_MasterQuestPlayer[questLogTitleText]["headerIsChecked"] = false;
			end

			QuestIon_QuestInLocation = false;
			QuestIon_tempLocationInfo = {};

			-- If so, recored the title for later use
			currentQuestLocation = questLogTitleText;

			QuestIon_tempLocationInfo["originalIndex"] = entryIndex;
			QuestIon_tempLocationInfo["titleText"] = questLogTitleText;
			QuestIon_tempLocationInfo["isHeader"] = isHeader;
		else
			-- Set defaults if the quest is new
			if (QuestIon_MasterQuestPlayer[questLogTitleText] == nil) then
				QuestIon_MasterQuestPlayer[questLogTitleText] = {};
			end
			if (QuestIon_MasterQuestPlayer[questLogTitleText]["isShown"] == nil) then
				QuestIon_MasterQuestPlayer[questLogTitleText]["isShown"] = QuestIon_MasterOptionPlayer["default_isShown"];
			end
			if (QuestIon_MasterQuestPlayer[questLogTitleText]["isComplete"] == nil) then
				if (isComplete) then
					QuestIon_MasterQuestPlayer[questLogTitleText]["isComplete"] = true;
				else
					QuestIon_MasterQuestPlayer[questLogTitleText]["isComplete"] = false;
				end
			end
			if (QuestIon_MasterQuestPlayer[questLogTitleText]["isShownComplete"] == nil) then
				QuestIon_MasterQuestPlayer[questLogTitleText]["isShownComplete"] = QuestIon_MasterOptionPlayer["default_isShownComplete"];
			end
			if (QuestIon_MasterQuestPlayer[questLogTitleText]["showObjectives"] == nil) then
				QuestIon_MasterQuestPlayer[questLogTitleText]["showObjectives"] = QuestIon_MasterOptionPlayer["default_showObjectives"];
			end
			if (QuestIon_MasterQuestPlayer[questLogTitleText]["showCompleteObjectives"] == nil) then
				QuestIon_MasterQuestPlayer[questLogTitleText]["showCompleteObjectives"] = QuestIon_MasterOptionPlayer["default_showCompleteObjectives"];
			end
			if (QuestIon_MasterQuestPlayer[questLogTitleText]["locationInTitle"] == nil) then
				QuestIon_MasterQuestPlayer[questLogTitleText]["locationInTitle"] = QuestIon_MasterOptionPlayer["default_locationInTitle"];
			end
			if (QuestIon_MasterQuestPlayer[questLogTitleText]["locationInObjectives"] == nil) then
				QuestIon_MasterQuestPlayer[questLogTitleText]["locationInObjectives"] = QuestIon_MasterOptionPlayer["default_locationInObjectives"];
			end
			if (QuestIon_MasterQuestPlayer[questLogTitleText]["showOnObjectivePickup"] == nil) then
				QuestIon_MasterQuestPlayer[questLogTitleText]["showOnObjectivePickup"] = QuestIon_MasterOptionPlayer["default_showOnObjectivePickup"];
			end
			if (QuestIon_MasterQuestPlayer[questLogTitleText]["showPartyInfo"] == nil) then
				QuestIon_MasterQuestPlayer[questLogTitleText]["showPartyInfo"] = QuestIon_MasterOptionPlayer["default_showPartyInfo"];
			end
			if (QuestIon_MasterQuestPlayer[questLogTitleText]["showNotOnQuest"] == nil) then
				QuestIon_MasterQuestPlayer[questLogTitleText]["showNotOnQuest"] = QuestIon_MasterOptionPlayer["default_showNotOnQuest"];
			end

			if (isComplete and QuestIon_MasterQuestPlayer[questLogTitleText]["isComplete"] == false) then
				UIErrorsFrame:AddMessage(questLogTitleText .. " (" .. COMPLETE .. ")", 1.0, 1.0, 0.0, 1.0, 4);
				if (QuestIon_MasterOptionPlayer["playSoundQuestComplete"]) then
					PlaySoundFile(QUESTIONTEXT_SOUND_QUESTCOMPLETE);
				end
			end

			if (isComplete) then
				QuestIon_MasterQuestPlayer[questLogTitleText]["isComplete"] = true;
			else
				QuestIon_MasterQuestPlayer[questLogTitleText]["isComplete"] = false;
			end
			if (not QuestIon_MasterQuestPlayer[questLogTitleText]["isShownComplete"] and isComplete) then
				QuestIon_MasterQuestPlayer[questLogTitleText]["isShown"] = false;
			end

			if (filterByLevel or filterByLocation) then
				QuestIon_MasterQuestPlayer[questLogTitleText]["isShown"] = false;
				if (filterByLevel and ( (UnitLevel("player") - filterRange) <= level and (UnitLevel("player") + filterRange) >= level ) ) then
					QuestIon_MasterQuestPlayer[questLogTitleText]["isShown"] = true;
				end
				if (filterByLocation and (GetRealZoneText() == currentQuestLocation or GetZoneText() == currentQuestLocation or GetSubZoneText() == currentQuestLocation or GetMinimapZoneText() == currentQuestLocation)) then
					QuestIon_MasterQuestPlayer[questLogTitleText]["isShown"] = true;
				end
			end

			-- Only add the quest to the array to sort if it is shown
			if (QuestIon_MasterQuestPlayer[questLogTitleText]["isShown"]) then
				if (QuestIon_QuestInLocation == false and sortMode == "normal" and QuestIon_MasterOptionPlayer["showLocationHeaders"]) then
					QuestIon_QuestInLocation = true;
					QuestIon_QuestHeaderIndex = QuestIon_QuestHeaderIndex + 1;
					QuestIon_QuestHeaderArray[QuestIon_QuestHeaderIndex] = {};
					QuestIon_QuestHeaderArray[QuestIon_QuestHeaderIndex]["originalIndex"] = QuestIon_tempLocationInfo["originalIndex"];
					QuestIon_QuestHeaderArray[QuestIon_QuestHeaderIndex]["titleText"] = QuestIon_tempLocationInfo["titleText"];
					QuestIon_QuestHeaderArray[QuestIon_QuestHeaderIndex]["isHeader"] = QuestIon_tempLocationInfo["isHeader"];
				end				
				if (not QuestIon_MasterQuestPlayer[currentQuestLocation]["headerIsMinimized"] or not QuestIon_MasterOptionPlayer["showLocationHeaders"]) then
					QuestIon_QuestHeaderIndex = QuestIon_QuestHeaderIndex + 1;
					QuestIon_QuestHeaderArray[QuestIon_QuestHeaderIndex] = {};
					QuestIon_QuestHeaderArray[QuestIon_QuestHeaderIndex]["originalIndex"] = entryIndex;
					QuestIon_QuestHeaderArray[QuestIon_QuestHeaderIndex]["titleText"] = questLogTitleText;
					QuestIon_QuestHeaderArray[QuestIon_QuestHeaderIndex]["level"] = level;
					QuestIon_QuestHeaderArray[QuestIon_QuestHeaderIndex]["questTag"] = questTag;
					QuestIon_QuestHeaderArray[QuestIon_QuestHeaderIndex]["isComplete"] = isComplete;
					QuestIon_QuestHeaderArray[QuestIon_QuestHeaderIndex]["location"] = currentQuestLocation;
				end
			end
		end
	end

	if (sortMode == "level") then
		table.sort(QuestIon_QuestHeaderArray, function (a,b)
			return (a["level"] < b["level"])
		end)
	elseif (sortMode == "alpha") then
		table.sort(QuestIon_QuestHeaderArray, function (a,b)
			return (a["titleText"] < b["titleText"])
		end)
	end

	local partyMemberShown = false;

	for entryIndex = 1, QuestIon_QuestHeaderIndex, 1 do
		-- Get several elements from the UI/XML
		local QuestIon_QuestText = getglobal("QuestIon_QuestTitle_" .. entryIndex);
		local QuestIon_QuestTextForWidth = getglobal("QuestIon_QuestTitle_" .. entryIndex .. "_Text");
		local QuestIon_QuestPlainTitle = getglobal("QuestIon_QuestTitle_" .. entryIndex .. "_PlainTitle");
		local QuestIon_QuestTag = getglobal("QuestIon_QuestTitle_" .. entryIndex .. "_Tag");
		local QuestIon_QuestObjectives = getglobal("QuestIon_QuestTitle_" .. entryIndex .. "_Objectives");

		-- Set the ID of the button to be the Quest Log index (Used in QuestIon_QuestTitle_OnClick)
		QuestIon_QuestText:SetID(QuestIon_QuestHeaderArray[entryIndex]["originalIndex"]);

		-- Is the current entry a Header?
		if (QuestIon_QuestHeaderArray[entryIndex]["isHeader"]) then
			-- Set title, clear objectives and size things

			-- Show/hide/clear the needed UI/XML elements
			QuestIon_QuestObjectives:SetHeight(0);
			QuestIon_QuestText:Show();
			QuestIon_QuestText:Enable();
			QuestIon_QuestTag:SetText("");
			QuestIon_QuestTag:Hide();
			QuestIon_QuestObjectives:SetText("");
			QuestIon_QuestObjectives:Hide();

			if (QuestIon_MasterQuestPlayer[QuestIon_QuestHeaderArray[entryIndex]["titleText"]]["headerIsMinimized"]) then
				QuestIon_QuestText:SetText("|cffffffff<" .. QuestIon_QuestHeaderArray[entryIndex]["titleText"] .. ">|r");
			else
				QuestIon_QuestText:SetText("|cffffffff" .. QuestIon_QuestHeaderArray[entryIndex]["titleText"] .. "|r");
			end
			QuestIon_QuestPlainTitle:SetText(QuestIon_QuestHeaderArray[entryIndex]["titleText"]);
			QuestIon_QuestText:SetWidth(QuestIon_QuestTextForWidth:GetStringWidth());
			QuestIon_QuestText:SetHeight(QuestIon_QuestTextForWidth:GetHeight());
			QuestIon_QuestTag:SetWidth(0);

			local currentWidth = QuestIon_QuestTextForWidth:GetStringWidth() + QuestIon_QuestTag:GetStringWidth();
			if (currentWidth > QuestIon_maxWidth) then
				QuestIon_maxWidth = currentWidth;
			end

		-- Is the current entry a Quest? (Not a Header)
		else
			-- Select the current quest
			SelectQuestLogEntry(QuestIon_QuestHeaderArray[entryIndex]["originalIndex"]);
			local questLogTitleText = QuestIon_QuestHeaderArray[entryIndex]["titleText"];
			local questTag = QuestIon_QuestHeaderArray[entryIndex]["questTag"];

			-- Show the needed UI/XML elements
			QuestIon_QuestObjectives:SetHeight(0);
			QuestIon_QuestText:Show();
			QuestIon_QuestText:Enable();
			QuestIon_QuestTag:Show();
			QuestIon_QuestObjectives:Show();

			-- Is the quest complete?
			if (QuestIon_MasterQuestPlayer[questLogTitleText]["isComplete"] ) then
				-- If so, set the questTag
				questTag = COMPLETE;
			end

			-- Is the quest failed?
			if ( IsCurrentQuestFailed() ) then
				-- If so, set the questTag
				questTag = FAILED;
			end

			if (QuestIon_MasterQuestPlayer[questLogTitleText]["locationInTitle"]) then
				if (questTag ~= nil) then
					questTag = "(" .. questTag .. ") - " .. QuestIon_QuestHeaderArray[entryIndex]["location"] .. "";
				else
					questTag = "- " .. QuestIon_QuestHeaderArray[entryIndex]["location"] .. "";
				end
			else
				if (questTag ~= nil) then
					questTag = "(" .. questTag .. ")";
				end
			end
			
			-- Get the Quest Title color using Blizzard's function from QuestLogFrame.lua
			local levelColor = GetDifficultyColor(QuestIon_QuestHeaderArray[entryIndex]["level"]);

			-- DEBUG:
			if (QuestIon_QuestHeaderArray[entryIndex]["isComplete"] and not QuestIon_MasterOptionPlayer["show_completeTag"]) then
				levelColor = { r = QuestIon_MasterOptionPlayer["color_complete_quest_red"] / 255, g = QuestIon_MasterOptionPlayer["color_complete_quest_green"] / 255, b = QuestIon_MasterOptionPlayer["color_complete_quest_blue"] / 255};
				questTag = nil;
			end
			-- :DEBUG

			-- Convert the levelColor to the |caarrggbb format
			local formattedLevelColor = string.format("|cff%02x%02x%02x", levelColor.r * 255, levelColor.g * 255, levelColor.b * 255);

			-- Display Quest Title line with colors and such
			QuestIon_QuestText:SetText(formattedLevelColor .. " [" .. QuestIon_QuestHeaderArray[entryIndex]["level"] .. "] " .. questLogTitleText .. "|r");
			QuestIon_QuestPlainTitle:SetText(questLogTitleText);

			-- Is there a questTag? (Complete, Failed, or Elite)
			if (questTag ~= nil) then
				-- If so, display the quest information and the quest tag
				QuestIon_QuestTag:SetText(questTag);
			else
				QuestIon_QuestTag:SetText("");
			end

			local QuestIon_questPercentComplete = 0;
			local numObjectives = GetNumQuestLeaderBoards();
			local QuestIon_QuestObjectivesText = "";

			if (QuestIon_MasterQuestPlayer[questLogTitleText]["showPartyInfo"]) then
				local numPartyMembers = GetNumPartyMembers();
				if (numPartyMembers ~= 0) then
					for i=1, numPartyMembers do
						isOnQuest = IsUnitOnQuest( QuestIon_QuestHeaderArray[entryIndex]["originalIndex"], "party"..i);
						unitName = UnitName("party"..i);
						if ( isOnQuest and isOnQuest == 1 ) then
							local formattedColor = string.format("|cff%02x%02x%02x", obj_end_red, obj_end_green, obj_end_blue);
							QuestIon_QuestObjectivesText = QuestIon_QuestObjectivesText .. " " .. formattedColor .. unitName .. "|r";
							if (i ~= numPartyMembers) then
								QuestIon_QuestObjectivesText = QuestIon_QuestObjectivesText .. ", ";
							end
							partyMemberShown = true;
						elseif (QuestIon_MasterQuestPlayer[questLogTitleText]["showNotOnQuest"]) then
							local formattedColor = string.format("|cff%02x%02x%02x", obj_start_red, obj_start_green, obj_start_blue);
							QuestIon_QuestObjectivesText = QuestIon_QuestObjectivesText .. formattedColor .. unitName .. "|r";
							if (i ~= numPartyMembers) then
								QuestIon_QuestObjectivesText = QuestIon_QuestObjectivesText .. ", ";
							end
							partyMemberShown = true;
						end
					end
					if (partyMemberShown) then
						QuestIon_QuestObjectivesText = QuestIon_QuestObjectivesText .. "\n";
					end
				end
			end

			if (QuestIon_MasterQuestPlayer[questLogTitleText]["locationInObjectives"]) then
				QuestIon_QuestObjectivesText = QuestIon_QuestObjectivesText .. " " .. QuestIon_QuestHeaderArray[entryIndex]["location"] .. "\n";
			end

			if (numObjectives ~= 0) then
				for objectiveIndex = 1, numObjectives, 1 do
					local text, quest_type, finished = GetQuestLogLeaderBoard(objectiveIndex);
					if ( not text or strlen(text) == 0 ) then
						text = quest_type;
					end
					if (finished) then
						if (QuestIon_MasterQuestPlayer[questLogTitleText]["showCompleteObjectives"]) then
							local formattedColor = string.format("|cff%02x%02x%02x", obj_end_red, obj_end_green, obj_end_blue);
							if (QuestIon_MasterQuestPlayer[questLogTitleText]["showObjectives"]) then
								QuestIon_QuestObjectivesText = QuestIon_QuestObjectivesText .. formattedColor .. " -" .. text .. " (" .. QUESTIONTEXT_DONE .. ")|r\n";
							end
						end
						QuestIon_questPercentComplete = QuestIon_questPercentComplete + 100/numObjectives;
					elseif (not finished) then
						local colon = string.find(text, ":");
						local percent = "0";
						if (colon) then
							local slash = string.find(text, "/");
							local length = string.len(text);
							local have = string.sub(text, colon + 2, slash - 1);
							local need = string.sub(text, slash + 1, length);
							if (type(tonumber(have)) == "number") then
								percent = string.format ("%d", have / need * 100);
							end
							if (percent == "0") then
								local formattedColor = string.format("|cff%02x%02x%02x", obj_start_red, obj_start_green, obj_start_blue);
								if (QuestIon_MasterQuestPlayer[questLogTitleText]["showObjectives"]) then
									QuestIon_QuestObjectivesText = QuestIon_QuestObjectivesText .. formattedColor .. " -" .. text .. "|r\n";
								end
							else
								local formattedColor = string.format("|cff%02x%02x%02x", obj_start_red - (obj_start_red - obj_end_red) * percent/100, obj_start_green - (obj_start_green - obj_end_green) * percent/100, obj_start_blue - (obj_start_blue - obj_end_blue) * percent/100);
								if (QuestIon_MasterQuestPlayer[questLogTitleText]["showObjectives"]) then
									QuestIon_QuestObjectivesText = QuestIon_QuestObjectivesText .. formattedColor .. " -" .. text .. " (" .. percent .. "%)|r\n";
								end
								QuestIon_questPercentComplete = QuestIon_questPercentComplete + percent/numObjectives;
							end
						else
							local formattedColor = string.format("|cff%02x%02x%02x", obj_start_red, obj_start_green, obj_start_blue);
							if (QuestIon_MasterQuestPlayer[questLogTitleText]["showObjectives"]) then
								QuestIon_QuestObjectivesText = QuestIon_QuestObjectivesText .. formattedColor .. " -" .. text .. "|r\n";
							end
						end							
					end
				end
				QuestIon_QuestObjectives:SetText(QuestIon_QuestObjectivesText);
				if (QuestIon_QuestObjectives:GetStringWidth() > QuestIon_maxWidth) then
					QuestIon_maxWidth = QuestIon_QuestObjectives:GetStringWidth();
				end
			elseif (QuestIon_MasterQuestPlayer[questLogTitleText]["showObjectives"] and not QuestIon_MasterOptionPlayer["hideQuestDescriptions"]) then
				local questDescription, questObjectives = GetQuestLogQuestText();
				local formattedColor = string.format("|cff%02x%02x%02x", obj_start_red, obj_start_green, obj_start_blue);

				QuestIon_QuestObjectives:SetText(QuestIon_QuestObjectivesText);

				if (QuestIon_QuestObjectives:GetStringWidth() > QuestIon_maxWidth) then
					QuestIon_maxWidth = QuestIon_QuestObjectives:GetStringWidth();
				end

				QuestIon_QuestObjectivesText = QuestIon_QuestObjectivesText .. formattedColor .. " -" .. questObjectives .. "|r\n";
				QuestIon_QuestObjectives:SetText(QuestIon_QuestObjectivesText);
			else
				QuestIon_QuestObjectives:SetText(QuestIon_QuestObjectivesText);
				if (QuestIon_QuestObjectives:GetStringWidth() > QuestIon_maxWidth) then
					QuestIon_maxWidth = QuestIon_QuestObjectives:GetStringWidth();
				end
			end

			QuestIon_questPercentComplete = math.floor(QuestIon_questPercentComplete + 0.5);
			if (QuestIon_questPercentComplete > 0 and QuestIon_questPercentComplete < 100) then
				if (QuestIon_QuestTag:GetText() ~= nil) then
					QuestIon_QuestTag:SetText("(" .. QuestIon_questPercentComplete .. "%) " .. QuestIon_QuestTag:GetText());
				else
					QuestIon_QuestTag:SetText("(" .. QuestIon_questPercentComplete .. "%)");
				end
			end

			QuestIon_QuestText:SetWidth(QuestIon_QuestTextForWidth:GetStringWidth());
			QuestIon_QuestText:SetHeight(QuestIon_QuestTextForWidth:GetHeight());
			QuestIon_QuestTag:SetWidth(0);
			QuestIon_QuestTag:SetWidth(QuestIon_QuestTag:GetStringWidth());
			QuestIon_QuestTag:SetHeight(QuestIon_QuestTextForWidth:GetHeight());
			local currentWidth = QuestIon_QuestTextForWidth:GetStringWidth() + QuestIon_QuestTag:GetStringWidth();
			if (currentWidth > QuestIon_maxWidth) then
				QuestIon_maxWidth = currentWidth;
			end
		end
	end

	QuestIon_Frame:SetWidth(QuestIon_maxWidth + 15);

	for cleanupIndex = 1, QuestIon_numButtons, 1 do
		local QuestIon_QuestText = getglobal("QuestIon_QuestTitle_" .. cleanupIndex);
		local QuestIon_QuestObjectives = getglobal("QuestIon_QuestTitle_" .. cleanupIndex .. "_Objectives");
		local QuestIon_QuestTag = getglobal("QuestIon_QuestTitle_" .. cleanupIndex .. "_Tag");
		local QuestIon_QuestPlainTitle = getglobal("QuestIon_QuestTitle_" .. cleanupIndex .. "_PlainTitle");

		if (cleanupIndex > QuestIon_QuestHeaderIndex) then
			QuestIon_QuestText:SetText("");
			QuestIon_QuestPlainTitle:SetText("");
			QuestIon_QuestTag:SetText("");
			QuestIon_QuestObjectives:SetText("");
			QuestIon_QuestText:Hide();
			QuestIon_QuestTag:Hide();
			QuestIon_QuestObjectives:Hide();
		else
			QuestIon_QuestText:Show();
			QuestIon_QuestText:Enable();
			QuestIon_QuestTag:Show();
			QuestIon_QuestObjectives:Show();
			local QuestIon_QuestTextForWidth = getglobal("QuestIon_QuestTitle_" .. cleanupIndex .. "_Text");
			QuestIon_QuestObjectives:SetWidth(QuestIon_maxWidth);
			if (QuestIon_QuestObjectives:GetStringWidth() > QuestIon_maxWidth) then
				local height = QuestIon_MasterOptionInfo["font_objective_size"]*math.ceil(QuestIon_QuestObjectives:GetStringWidth() / QuestIon_maxWidth);
				if (QuestIon_MasterQuestPlayer[QuestIon_QuestPlainTitle:GetText()]["locationInObjectives"]) then
					height = height + QuestIon_MasterOptionInfo["font_objective_size"];
				end
				local numPartyMembers = GetNumPartyMembers();
				if (partyMemberShown) then
					height = height + QuestIon_MasterOptionInfo["font_objective_size"];
				end
				QuestIon_QuestObjectives:SetHeight(height);
			end
			QuestIon_totalHeight = QuestIon_totalHeight + QuestIon_QuestTextForWidth:GetHeight() + QuestIon_QuestObjectives:GetHeight() + 1;
		end
	end

	if (QuestIon_QuestHeaderIndex == 0) then
		local QuestIon_QuestObjectives = getglobal("QuestIon_QuestTitle_1_Objectives");
		local QuestIon_QuestText = getglobal("QuestIon_QuestTitle_1");
		local QuestIon_QuestTextForWidth = getglobal("QuestIon_QuestTitle_1_Text");

		-- Show/hide/clear the needed UI/XML elements
		QuestIon_QuestObjectives:SetHeight(0);
		QuestIon_QuestText:Show();
		QuestIon_QuestText:Disable();

		QuestIon_QuestText:SetText("|cff999999" .. QUESTIONTEXT_NOQUESTSHOWN .. "|r");
		QuestIon_QuestText:SetWidth(QuestIon_QuestTextForWidth:GetStringWidth());
		QuestIon_QuestText:SetHeight(QuestIon_QuestTextForWidth:GetHeight());

		local currentWidth = QuestIon_QuestTextForWidth:GetStringWidth();
		if (currentWidth > QuestIon_maxWidth) then
			QuestIon_maxWidth = currentWidth;
		end
		QuestIon_Frame:SetWidth(QuestIon_maxWidth + 15);

		QuestIon_totalHeight = QuestIon_QuestTextForWidth:GetHeight();
	end

	QuestIon_Frame:SetHeight(QuestIon_totalHeight + 30);

	SelectQuestLogEntry(questSelected);

	if (wasQuestLogVisible) then
		ShowUIPanel(QuestLogFrame);
	end

	if (not QuestIon_PastFirstUpdate) then
		QuestIon_PastFirstUpdate = true;
		if (QuestIon_MasterOptionPlayer["frame_minimized"]) then
			QuestIon_SetMinimized();
		end
		QuestIon_OnLeave();
		QuestIon_MouseUp();
		if (QuestIon_MasterOptionPlayer["frame_hidden"]) then
			QuestIon_SlashHandler("hide");
		end

		QuestIon_InternalQuestCleanup();

		-- Display the loaded message
		QuestIon_ChatPrint("|cff0080ffQuestIon v" .. QuestIon_VERSION .." loaded|r");
	end

	if (QuestIon_MasterOptionPlayer["frame_minimized"]) then
		QuestIon_SetMinimized();
	end

	if (numEntries == 0) then
		QuestIon_allowUpdates = true;
	else
		QuestIon_allowUpdates = false;
	end
end

function QuestIon_InfoMessageUpdate(ui_message)
	local colon = string.find(ui_message, ":");
	if (colon) then
		QuestIon_objectiveToFind = string.sub(ui_message, 1, colon);

		local numEntries, numQuests = GetNumQuestLogEntries()

		for index = 1, numEntries, 1 do
			-- Gather some information on our current Quest Log Entry
			local questLogTitleText, level, questTag, isHeader, isCollapsed, isComplete = GetQuestLogTitle(index);

			if (not isHeader) then
				SelectQuestLogEntry(index);
				local numObjectives = GetNumQuestLeaderBoards();
				if (numObjectives ~= 0) then
					for objectiveIndex = 1, numObjectives, 1 do
						local text, quest_type, finished = GetQuestLogLeaderBoard(objectiveIndex);
						if ( not text or strlen(text) == 0 ) then
							text = quest_type;
						end
						if (QuestIon_objectiveToFind == string.sub(text, 1, colon) and QuestIon_MasterQuestPlayer[questLogTitleText]["showOnObjectivePickup"]) then
							QuestIon_MasterQuestPlayer[questLogTitleText]["isShown"] = true;
							if (QuestIon_MasterOptionPlayer["unminimizeOnPickup"]) then
								QuestIon_SetRestore();
							end
						end
					end
				end	
			end
		end
		QuestIon_WindowUpdate();
	end
end

-----------------------------
-- Quest Title Drop Down Menu
-----------------------------

function QuestIon_QuestTitle_DropDown_OnLoad()
	UIDropDownMenu_Initialize(this, QuestIon_QuestTitle_DropDown_Initialize, "MENU");
end

function QuestIon_QuestTitle_DropDown_Initialize(level)
	-- Set the level to 1 is no level is given
	if (level == nil) then
		level = 1;
	end
	
	-- If we don't have a menu value then exit
	if (UIDROPDOWNMENU_MENU_VALUE == nil) then
		return;
	end

	-- Get the name of the quest that was clicked on
	local questClicked = getglobal(UIDROPDOWNMENU_MENU_VALUE.."_PlainTitle"):GetText();

	-- Track if the Quest Log was visible
	local wasQuestLogVisible = false;

	-- Check if the Quest Log was visible
	if ( QuestLogFrame:IsVisible() ) then
		-- Hide the Quest Log while we update
		HideUIPanel(QuestLogFrame);

		-- Record that the Quest Log was visible
		wasQuestLogVisible = true;
	end

	-- Record the currently selected quest in the Quest Log
	local questSelected = GetQuestLogSelection();

	-- Display the quest's name
	local info = {};
	info.text = "- |cff0080ff" .. questClicked .. "|r -";
	info.notClickable = 1;
	info.isTitle = 1;
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);

	-- Determine if questClicked is a location header
	if (QuestIon_MasterQuestPlayer[questClicked]["headerIsMinimized"] ~= nil) then
		local headerID = getglobal(UIDROPDOWNMENU_MENU_VALUE):GetID();

		local numEntries, numQuests = GetNumQuestLogEntries()

		-- Start with the item after the location header
		for entryIndex = headerID + 1, numEntries, 1 do
			local questLogTitleText, level, questTag, isHeader, isCollapsed, isComplete = GetQuestLogTitle(entryIndex);
			if (isHeader) then
				-- Quit at the next header
				break;
			else
				-- Is the quest complete?
				if ( isComplete ) then
					-- If so, set the questTag
					questTag = COMPLETE;
				end

				-- Is the quest failed?
				if ( IsCurrentQuestFailed() ) then
					-- If so, set the questTag
					questTag = FAILED;
				end

				info = {};
				-- Get the Quest Title color using Blizzard's function from QuestLogFrame.lua
				local levelColor = GetDifficultyColor(level);

				-- Convert the levelColor to the |caarrggbb format
				local formattedLevelColor = string.format("|cff%02x%02x%02x", levelColor.r * 255, levelColor.g * 255, levelColor.b * 255);
				info.text = formattedLevelColor .. "[" .. level .. "] " .. questLogTitleText .. "|r";
				info.value = questLogTitleText;
				if (QuestIon_MasterQuestPlayer[questLogTitleText]["isShown"]) then
					info.checked = 1;
				end
				info.func = QuestIon_FrameTitle_DropDown_ToggleShowQuest;
				info.keepShownOnClick = 1;

				-- Is there a questTag? (Complete, Failed, or Elite)
				if (questTag ~= nil) then
					info.tooltipTitle = formattedLevelColor .. questLogTitleText .. "|r (" .. questTag .. ")";
				else
					info.tooltipTitle = formattedLevelColor .. questLogTitleText .. "|r";
				end

				-- Select the quest to get the description
				SelectQuestLogEntry(entryIndex);
				local questDescription, questObjectives = GetQuestLogQuestText();
				info.tooltipText = "|cffffffff" .. questClicked .. "\n|r|cffcccccc -" .. questObjectives .. "|r";

				UIDropDownMenu_AddButton(info);	
			end
		end
	else
		local info = {};
		info.text = QUESTIONTEXT_DISPLAY;
		info.notClickable = 1;
		info.isTitle = 1;
		info.notCheckable = 1;
		UIDropDownMenu_AddButton(info);

		info = {};
		info.text = QUESTIONTEXT_HIDEQUEST;
		if(not QuestIon_MasterQuestPlayer[questClicked]["isShown"]) then
			info.checked = 1;
		end
		info.func = QuestIon_QuestTitle_DropDown_ToggleHideQuest;
		info.value = questClicked;
		info.keepShownOnClick = 1;
		UIDropDownMenu_AddButton(info);

		info = {};
		info.text = QUESTIONTEXT_SHOWONOBJPICKUP;
		if(QuestIon_MasterQuestPlayer[questClicked]["showOnObjectivePickup"]) then
			info.checked = 1;
		end
		info.func = QuestIon_QuestTitle_DropDown_ToggleObjectivePickup;
		info.value = questClicked;
		info.keepShownOnClick = 1;
		UIDropDownMenu_AddButton(info);

		info = {};
		info.text = QUESTIONTEXT_HIDEWHENCOMPLETE;
		if(not QuestIon_MasterQuestPlayer[questClicked]["isShownComplete"]) then
			info.checked = 1;
		end
		info.func = QuestIon_QuestTitle_DropDown_ToggleCompleteQuest;
		info.value = questClicked;
		info.keepShownOnClick = 1;
		UIDropDownMenu_AddButton(info);
	
		local info = {};
		info.text = QUESTIONTEXT_OBJECTIVES;
		info.notClickable = 1;
		info.isTitle = 1;
		info.notCheckable = 1;
		UIDropDownMenu_AddButton(info);

		info = {};
		info.text = QUESTIONTEXT_SHOWOBJECTIVES;
		if(QuestIon_MasterQuestPlayer[questClicked]["showObjectives"]) then
			info.checked = 1;
		end
		info.func = QuestIon_QuestTitle_DropDown_ToggleObjectives;
		info.value = questClicked;
		info.keepShownOnClick = 1;
		UIDropDownMenu_AddButton(info);

		info = {};
		info.text = QUESTIONTEXT_SHOWFINISHEDOBJ;
		if(QuestIon_MasterQuestPlayer[questClicked]["showCompleteObjectives"]) then
			info.checked = 1;
		end
		info.func = QuestIon_QuestTitle_DropDown_ToggleCompleteObjectives;
		info.value = questClicked;
		info.keepShownOnClick = 1;
		UIDropDownMenu_AddButton(info);

		local info = {};
		info.text = QUESTIONTEXT_LOCATION;
		info.notClickable = 1;
		info.isTitle = 1;
		info.notCheckable = 1;
		UIDropDownMenu_AddButton(info);

		info = {};
		info.text = QUESTIONTEXT_LOCATIONINTITLE;
		if(QuestIon_MasterQuestPlayer[questClicked]["locationInTitle"]) then
			info.checked = 1;
		end
		info.func = QuestIon_QuestTitle_DropDown_TogglelocationInTitle;
		info.value = questClicked;
		info.keepShownOnClick = 1;
		UIDropDownMenu_AddButton(info);

		info = {};
		info.text = QUESTIONTEXT_LOCATIONINOBJECTIVES;
		if(QuestIon_MasterQuestPlayer[questClicked]["locationInObjectives"]) then
			info.checked = 1;
		end
		info.func = QuestIon_QuestTitle_DropDown_TogglelocationInObjectives;
		info.value = questClicked;
		info.keepShownOnClick = 1;
		UIDropDownMenu_AddButton(info);

		local info = {};
		info.text = QUESTIONTEXT_PARTYSTATUS;
		info.notClickable = 1;
		info.isTitle = 1;
		info.notCheckable = 1;
		UIDropDownMenu_AddButton(info);

		info = {};
		info.text = QUESTIONTEXT_SHOWPARTYSTATUS;
		if(QuestIon_MasterQuestPlayer[questClicked]["showPartyInfo"]) then
			info.checked = 1;
		end
		info.func = QuestIon_QuestTitle_DropDown_ToggleshowPartyInfo;
		info.value = questClicked;
		info.keepShownOnClick = 1;
		UIDropDownMenu_AddButton(info);

		info = {};
		info.text = QUESTIONTEXT_HIDEPARTYNOQUEST;
		if(not QuestIon_MasterQuestPlayer[questClicked]["showNotOnQuest"]) then
			info.checked = 1;
		end
		info.func = QuestIon_QuestTitle_DropDown_ToggleshowNotOnQuest;
		info.value = questClicked;
		info.keepShownOnClick = 1;
		UIDropDownMenu_AddButton(info);
	end

	info = {};
	info.text = QUESTIONTEXT_EXITMENU;
	info.func = QuestIon_DropDown_ExitMenu;
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);

	SelectQuestLogEntry(questSelected);

	if (wasQuestLogVisible) then
		ShowUIPanel(QuestLogFrame);
	end
end

function QuestIon_QuestTitle_DropDown_ToggleHideQuest()
	if (QuestIon_MasterQuestPlayer[this.value]["isShown"]) then
		QuestIon_MasterQuestPlayer[this.value]["isShown"] = false;
	else
		QuestIon_MasterQuestPlayer[this.value]["isShown"] = true;
		if (QuestIon_MasterQuestPlayer[this.value]["isComplete"]) then
			QuestIon_MasterQuestPlayer[this.value]["isShownComplete"] = true;
		end
	end
	QuestIon_WindowUpdate();
end

function QuestIon_QuestTitle_DropDown_ToggleCompleteQuest()
	if (QuestIon_MasterQuestPlayer[this.value]["isShownComplete"]) then
		QuestIon_MasterQuestPlayer[this.value]["isShownComplete"] = false;
	else
		QuestIon_MasterQuestPlayer[this.value]["isShownComplete"] = true;
	end
	QuestIon_WindowUpdate();
end

function QuestIon_QuestTitle_DropDown_ToggleObjectivePickup()
	if (QuestIon_MasterQuestPlayer[this.value]["showOnObjectivePickup"]) then
		QuestIon_MasterQuestPlayer[this.value]["showOnObjectivePickup"] = false;
	else
		QuestIon_MasterQuestPlayer[this.value]["showOnObjectivePickup"] = true;
	end
end

function QuestIon_QuestTitle_DropDown_ToggleObjectives()
	if (QuestIon_MasterQuestPlayer[this.value]["showObjectives"]) then
		QuestIon_MasterQuestPlayer[this.value]["showObjectives"] = false;
	else
		QuestIon_MasterQuestPlayer[this.value]["showObjectives"] = true;
	end
	QuestIon_WindowUpdate();
end

function QuestIon_QuestTitle_DropDown_ToggleCompleteObjectives()
	if (QuestIon_MasterQuestPlayer[this.value]["showCompleteObjectives"]) then
		QuestIon_MasterQuestPlayer[this.value]["showCompleteObjectives"] = false;
	else
		QuestIon_MasterQuestPlayer[this.value]["showCompleteObjectives"] = true;
	end
	QuestIon_WindowUpdate();
end

function QuestIon_QuestTitle_DropDown_TogglelocationInTitle()
	if (QuestIon_MasterQuestPlayer[this.value]["locationInTitle"]) then
		QuestIon_MasterQuestPlayer[this.value]["locationInTitle"] = false;
	else
		QuestIon_MasterQuestPlayer[this.value]["locationInTitle"] = true;
	end
	QuestIon_WindowUpdate();
end

function QuestIon_QuestTitle_DropDown_TogglelocationInObjectives()
	if (QuestIon_MasterQuestPlayer[this.value]["locationInObjectives"]) then
		QuestIon_MasterQuestPlayer[this.value]["locationInObjectives"] = false;
	else
		QuestIon_MasterQuestPlayer[this.value]["locationInObjectives"] = true;
	end
	QuestIon_WindowUpdate();
end

function QuestIon_QuestTitle_DropDown_ToggleshowPartyInfo()
	if (QuestIon_MasterQuestPlayer[this.value]["showPartyInfo"]) then
		QuestIon_MasterQuestPlayer[this.value]["showPartyInfo"] = false;
	else
		QuestIon_MasterQuestPlayer[this.value]["showPartyInfo"] = true;
	end
	QuestIon_WindowUpdate();
end

function QuestIon_QuestTitle_DropDown_ToggleshowNotOnQuest()
	if (QuestIon_MasterQuestPlayer[this.value]["showNotOnQuest"]) then
		QuestIon_MasterQuestPlayer[this.value]["showNotOnQuest"] = false;
	else
		QuestIon_MasterQuestPlayer[this.value]["showNotOnQuest"] = true;
	end
	QuestIon_WindowUpdate();
end

-----------------------------
-- Frame Title Drop Down Menu
-----------------------------

function QuestIon_FrameTitle_DropDown_OnLoad()
	UIDropDownMenu_Initialize(this, QuestIon_FrameTitle_DropDown_Initialize, "MENU");
end

function QuestIon_FrameTitle_DropDown_Initialize(level)
	if (level == nil) then
		level = 1;
	end

	if (level == 1) then
		local info = {};
		info.text = QUESTIONTEXT_SHOWHIDEQUESTS;
		info.hasArrow = 1;
		info.notCheckable = 1;
		UIDropDownMenu_AddButton(info);

		info = {};
		info.notClickable = 1;
		UIDropDownMenu_AddButton(info);

		info = {};
		info.text = QUESTIONTEXT_SORTINGFILTERING;
		info.hasArrow = 1;
		info.notCheckable = 1;
		UIDropDownMenu_AddButton(info);

		info = {};
		info.text = QUESTIONTEXT_NEWQUESTOPTIONS;
		info.hasArrow = 1;
		info.notCheckable = 1;
		UIDropDownMenu_AddButton(info);

		info = {};
		info.text = QUESTIONTEXT_GLOBALCOMMANDS;
		info.hasArrow = 1;
		info.notCheckable = 1;
		UIDropDownMenu_AddButton(info);

		info = {};
		info.notClickable = 1;
		UIDropDownMenu_AddButton(info);

		info = {};
		info.text = QUESTIONTEXT_COLORSFONTS;
		info.hasArrow = 1;
		info.notCheckable = 1;
		UIDropDownMenu_AddButton(info);

		info = {};
		info.text = QUESTIONTEXT_FADINGBEHAVIOR;
		info.hasArrow = 1;
		info.notCheckable = 1;
		UIDropDownMenu_AddButton(info);

		info = {};
		info.text = QUESTIONTEXT_SHOWRESIZING;
		info.hasArrow = 1;
		info.notCheckable = 1;
		UIDropDownMenu_AddButton(info);

		info = {};
		info.text = QUESTIONTEXT_MISCOPTIONS;
		info.hasArrow = 1;
		info.notCheckable = 1;
		UIDropDownMenu_AddButton(info);

		info = {};
		if (QuestIon_MasterOptionPlayer ~= nil and not QuestIon_MasterOptionPlayer["windowLocked"]) then
			info.text = LOCK_WINDOW;
		else
			info.text = UNLOCK_WINDOW;
		end
		info.func = QuestIon_DropDown_ToggleLock;
		info.notCheckable = 1;
		UIDropDownMenu_AddButton(info);

		info = {};
		info.text = QUESTIONTEXT_EXITMENU;
		info.func = QuestIon_DropDown_ExitMenu;
		info.notCheckable = 1;
		UIDropDownMenu_AddButton(info);

	elseif (level ==2) then
		if (UIDROPDOWNMENU_MENU_VALUE == QUESTIONTEXT_SHOWHIDEQUESTS) then
			QuestIon_WindowUpdate();
			-- Get how many entries (quests + headers) and quests are in the Quest Log
			local numEntries, numQuests = GetNumQuestLogEntries()
			local currentQuestLocation = "";

			if (numEntries > UIDROPDOWNMENU_MAXBUTTONS - 2) then
				numEntries = UIDROPDOWNMENU_MAXBUTTONS - 2;
				QuestIon_enableSecondDropdownMenu = true;
			else
				QuestIon_enableSecondDropdownMenu = false;
			end

			-- Go through the entire list in the Quest Log
			for entryIndex = 1, numEntries, 1 do
				-- Gather some information on our current Quest Log Entry
				local questLogTitleText, level, questTag, isHeader, isCollapsed, isComplete = GetQuestLogTitle(entryIndex);

				-- Is the current entry a Header?
				if (not isHeader) then
					SelectQuestLogEntry(entryIndex);
					
					-- Is the quest complete?
					if ( isComplete ) then
						-- If so, set the questTag
						questTag = COMPLETE;
					end

					-- Is the quest failed?
					if ( IsCurrentQuestFailed() ) then
						-- If so, set the questTag
						questTag = FAILED;
					end

					info = {};
					-- Get the Quest Title color using Blizzard's function from QuestLogFrame.lua
					local levelColor = GetDifficultyColor(level);

					-- Convert the levelColor to the |caarrggbb format
					local formattedLevelColor = string.format("|cff%02x%02x%02x", levelColor.r * 255, levelColor.g * 255, levelColor.b * 255);
					info.text = formattedLevelColor .. "  [" .. level .. "] " .. questLogTitleText .. "|r";
					info.value = questLogTitleText;
					if (QuestIon_MasterQuestPlayer[questLogTitleText]["isShown"]) then
						info.checked = 1;
					end
					info.func = QuestIon_FrameTitle_DropDown_ToggleShowQuest;
					info.keepShownOnClick = 1;

					-- Is there a questTag? (Complete, Failed, or Elite)
					if (questTag ~= nil) then
						info.tooltipTitle = formattedLevelColor .. questLogTitleText .. "|r (" .. questTag .. ")";
					else
						info.tooltipTitle = formattedLevelColor .. questLogTitleText .. "|r";
					end
					local questDescription, questObjectives = GetQuestLogQuestText();
					info.tooltipText = "|cffffffff" .. currentQuestLocation .. "\n|r|cffcccccc -" .. questObjectives .. "|r";

					UIDropDownMenu_AddButton(info,2);	
				else
					currentQuestLocation = questLogTitleText;
					QuestIon_secondDropdownMenuLocation = questLogTitleText;
					info = {};
					info.text = questLogTitleText;
					if (QuestIon_MasterQuestPlayer[questLogTitleText]["headerIsChecked"]) then
						info.checked = 1;
					end
					info.value = questLogTitleText;
					info.func = QuestIon_FrameTitle_DropDown_ToggleHeader;
					info.keepShownOnClick = 1;
					UIDropDownMenu_AddButton(info,2);
				end
			end

			if (QuestIon_enableSecondDropdownMenu) then
				info = {};
				info.text = QUESTIONTEXT_MOREQUESTS;
				info.hasArrow = 1;
				info.notCheckable = 1;
				UIDropDownMenu_AddButton(info, 2);
			end				

			info = {};
			info.text = QUESTIONTEXT_EXITMENU;
			info.func = QuestIon_DropDown_ExitMenu;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);	

		elseif (UIDROPDOWNMENU_MENU_VALUE == QUESTIONTEXT_SORTINGFILTERING) then
			local info = {};
			info.text = QUESTIONTEXT_SORTING;
			info.notClickable = 1;
			info.isTitle = 1;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_DEFAULTSORTORDER;
			info.func = QuestIon_FrameTitle_DropDown_default_sortMode;
			if (QuestIon_MasterOptionPlayer["default_sortMode"] == "normal") then
				info.checked = 1;
			end
			info.value = "normal";
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);	

			info = {};
			info.text = QUESTIONTEXT_LEVELSORTORDER;
			info.func = QuestIon_FrameTitle_DropDown_default_sortMode;
			if (QuestIon_MasterOptionPlayer["default_sortMode"] == "level") then
				info.checked = 1;
			end
			info.value = "level";
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_ALPHASORTORDER;
			info.func = QuestIon_FrameTitle_DropDown_default_sortMode;
			if (QuestIon_MasterOptionPlayer["default_sortMode"] == "alpha") then
				info.checked = 1;
			end
			info.value = "alpha";
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			local info = {};
			info.text = QUESTIONTEXT_FILTERING;
			info.notClickable = 1;
			info.isTitle = 1;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			local info = {};
			info.text = "|cffff0000" .. QUESTIONTEXT_FILTERNOTE .. "|r";
			info.notClickable = 1;
			info.isTitle = 1;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_FILTERLOCATION;
			info.func = QuestIon_FrameTitle_DropDown_default_filterByLocation;
			if (QuestIon_MasterOptionPlayer["default_filterByLocation"]) then
				info.checked = 1;
			end
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);	

			info = {};
			info.text = QUESTIONTEXT_FILTERBYLEVEL;
			info.func = QuestIon_FrameTitle_DropDown_default_filterByLevel;
			if (QuestIon_MasterOptionPlayer["default_filterByLevel"]) then
				info.checked = 1;
			end
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_LEVELRANGE .. " |cff00c0ff" .. QuestIon_MasterOptionPlayer["default_filterRange"] .. "|r " .. QUESTIONTEXT_CLICKTOCHANGE;
			info.func = QuestIon_FrameTitle_DropDown_default_filterRange;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_EXITMENU;
			info.func = QuestIon_DropDown_ExitMenu;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

		elseif (UIDROPDOWNMENU_MENU_VALUE == QUESTIONTEXT_NEWQUESTOPTIONS) then
			local info = {};
			info.text = QUESTIONTEXT_DISPLAY;
			info.notClickable = 1;
			info.isTitle = 1;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_SHOWNEWQUESTS;
			info.func = QuestIon_FrameTitle_DropDown_default_isShown;
			if (QuestIon_MasterOptionPlayer["default_isShown"]) then
				info.checked = 1;
			end
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);	

			info = {};
			info.text = QUESTIONTEXT_SHOWONOBJPICKUP;
			info.func = QuestIon_FrameTitle_DropDown_default_showOnObjectivePickup;
			if (QuestIon_MasterOptionPlayer["default_showOnObjectivePickup"]) then
				info.checked = 1;
			end
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_HIDEWHENCOMPLETE;
			info.func = QuestIon_FrameTitle_DropDown_default_isShownComplete;
			if (not QuestIon_MasterOptionPlayer["default_isShownComplete"]) then
				info.checked = 1;
			end
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);	

			info = {};
			info.text = QUESTIONTEXT_OBJECTIVES;
			info.notClickable = 1;
			info.isTitle = 1;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_SHOWOBJECTIVES;
			info.func = QuestIon_FrameTitle_DropDown_default_showObjectives;
			if (QuestIon_MasterOptionPlayer["default_showObjectives"]) then
				info.checked = 1;
			end
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);	

			info = {};
			info.text = QUESTIONTEXT_SHOWFINISHEDOBJ;
			info.func = QuestIon_FrameTitle_DropDown_default_showCompleteObjectives;
			if (QuestIon_MasterOptionPlayer["default_showCompleteObjectives"]) then
				info.checked = 1;
			end
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);	

			local info = {};
			info.text = QUESTIONTEXT_LOCATION;
			info.notClickable = 1;
			info.isTitle = 1;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_LOCATIONINTITLE;
			info.func = QuestIon_FrameTitle_DropDown_default_locationInTitle;
			if (QuestIon_MasterOptionPlayer["default_locationInTitle"]) then
				info.checked = 1;
			end
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);	

			info = {};
			info.text = QUESTIONTEXT_LOCATIONINOBJECTIVES;
			info.func = QuestIon_FrameTitle_DropDown_default_locationInObjectives;
			if (QuestIon_MasterOptionPlayer["default_locationInObjectives"]) then
				info.checked = 1;
			end
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);	

			local info = {};
			info.text = QUESTIONTEXT_PARTYSTATUS;
			info.notClickable = 1;
			info.isTitle = 1;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_SHOWPARTYSTATUS;
			info.func = QuestIon_FrameTitle_DropDown_default_showPartyInfo;
			if (QuestIon_MasterOptionPlayer["default_showPartyInfo"]) then
				info.checked = 1;
			end
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);	

			info = {};
			info.text = QUESTIONTEXT_HIDEPARTYNOQUEST;
			info.func = QuestIon_FrameTitle_DropDown_default_showNotOnQuest;
			if (not QuestIon_MasterOptionPlayer["default_showNotOnQuest"]) then
				info.checked = 1;
			end
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			local info = {};
			info.text = QUESTIONTEXT_SPECIAL;
			info.notClickable = 1;
			info.isTitle = 1;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_APPLYTOQUESTS;
			info.func = QuestIon_FrameTitle_DropDown_SetAllQuestsToDefault;
			info.notCheckable = 1;
			info.tooltipTitle = "Set all as new quests";
			info.tooltipText = "This will set all current quests to use the same options as new quests.";
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_EXITMENU;
			info.func = QuestIon_DropDown_ExitMenu;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);	

		elseif (UIDROPDOWNMENU_MENU_VALUE == QUESTIONTEXT_GLOBALCOMMANDS) then
			local info = {};
			info.text = "|cffff0000" .. QUESTIONTEXT_COMMANDNOTE .. "|r";
			info.notClickable = 1;
			info.isTitle = 1;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_DISPLAY;
			info.notClickable = 1;
			info.isTitle = 1;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_CMD_SHOWQUESTS;
			info.func = QuestIon_FrameTitle_DropDown_ShowAllQuests;
			info.notCheckable = 1;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_CMD_HIDEQUESTS;
			info.func = QuestIon_FrameTitle_DropDown_HideAllQuests;
			info.notCheckable = 1;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_CMD_SETOBJPICKUP;
			info.func = QuestIon_FrameTitle_DropDown_SetObjectivePickup;
			info.notCheckable = 1;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_CMD_UNSETOBJPICKUP;
			info.func = QuestIon_FrameTitle_DropDown_UnsetObjectivePickup;
			info.notCheckable = 1;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_CMD_SHOWFINISHEDQUESTS;
			info.func = QuestIon_FrameTitle_DropDown_ShowAllCompletedQuests;
			info.notCheckable = 1;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_CMD_HIDEFINISHEDQUESTS;
			info.func = QuestIon_FrameTitle_DropDown_HideAllCompletedQuests;
			info.notCheckable = 1;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			local info = {};
			info.text = QUESTIONTEXT_OBJECTIVES;
			info.notClickable = 1;
			info.isTitle = 1;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  "  .. QUESTIONTEXT_SHOWOBJECTIVES;
			info.func = QuestIon_FrameTitle_DropDown_ShowAllObjectives;
			info.notCheckable = 1;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_HIDEOBJECTIVES;
			info.func = QuestIon_FrameTitle_DropDown_HideAllObjectives;
			info.notCheckable = 1;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_SHOWFINISHEDOBJ;
			info.func = QuestIon_FrameTitle_DropDown_ShowAllCompleteObjectives;
			info.notCheckable = 1;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_HIDEFINISHEDOBJ;
			info.func = QuestIon_FrameTitle_DropDown_HideAllCompleteObjectives;
			info.notCheckable = 1;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			local info = {};
			info.text = QUESTIONTEXT_LOCATION;
			info.notClickable = 1;
			info.isTitle = 1;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_LOCATIONINTITLE;
			info.func = QuestIon_FrameTitle_DropDown_ShowLocationInTitles;
			info.notCheckable = 1;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_HIDELOCATIONINTITLE;
			info.func = QuestIon_FrameTitle_DropDown_HideLocationInTitles;
			info.notCheckable = 1;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_LOCATIONINOBJECTIVES;
			info.func = QuestIon_FrameTitle_DropDown_ShowLocationInObjectives;
			info.notCheckable = 1;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_HIDELOCATIONINOBJECTIVES;
			info.func = QuestIon_FrameTitle_DropDown_HideLocationInObjectives;
			info.notCheckable = 1;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			local info = {};
			info.text = QUESTIONTEXT_PARTYSTATUS;
			info.notClickable = 1;
			info.isTitle = 1;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_SHOWPARTYSTATUS;
			info.func = QuestIon_FrameTitle_DropDown_ShowPartyStatus;
			info.notCheckable = 1;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_HIDEPARTYSTATUS;
			info.func = QuestIon_FrameTitle_DropDown_HidePartyStatus;
			info.notCheckable = 1;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_SHOWALLPARTY;
			info.func = QuestIon_FrameTitle_DropDown_ShowAllInPartyStatus;
			info.notCheckable = 1;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_HIDEPARTYNOQUEST;
			info.func = QuestIon_FrameTitle_DropDown_HidePartyNoQuest;
			info.notCheckable = 1;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_EXITMENU;
			info.func = QuestIon_DropDown_ExitMenu;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);	

		elseif (UIDROPDOWNMENU_MENU_VALUE == QUESTIONTEXT_COLORSFONTS) then
			local info = {};
			info.text = QUESTIONTEXT_COLORS;
			info.notClickable = 1;
			info.isTitle = 1;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_STARTOBJCOLOR;
			info.hasColorSwatch = 1;
			info.r = QuestIon_MasterOptionPlayer["color_obj_start_red"]/255;
			info.g = QuestIon_MasterOptionPlayer["color_obj_start_green"]/255;
			info.b = QuestIon_MasterOptionPlayer["color_obj_start_blue"]/255;
			info.swatchFunc = QuestIon_ColorPicker_Start_Objective;
			info.func = UIDropDownMenuButton_OpenColorPicker;
			info.cancelFunc = QuestIon_ColorPickerCancel_Start_Objective;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);	

			info = {};
			info.text = "  " .. QUESTIONTEXT_ENDOBJCOLOR;
			info.hasColorSwatch = 1;
			info.r = QuestIon_MasterOptionPlayer["color_obj_end_red"]/255;
			info.g = QuestIon_MasterOptionPlayer["color_obj_end_green"]/255;
			info.b = QuestIon_MasterOptionPlayer["color_obj_end_blue"]/255;
			info.swatchFunc = QuestIon_ColorPicker_End_Objective;
			info.func = UIDropDownMenuButton_OpenColorPicker;
			info.cancelFunc = QuestIon_ColorPickerCancel_End_Objective;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_COMPLETEQUESTCOLOR;
			info.hasColorSwatch = 1;
			info.r = QuestIon_MasterOptionPlayer["color_complete_quest_red"]/255;
			info.g = QuestIon_MasterOptionPlayer["color_complete_quest_green"]/255;
			info.b = QuestIon_MasterOptionPlayer["color_complete_quest_blue"]/255;
			info.swatchFunc = QuestIon_ColorPicker_Complete_Quest;
			info.func = UIDropDownMenuButton_OpenColorPicker;
			info.cancelFunc = QuestIon_ColorPickerCancel_Complete_Quest;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_COMPLETEQUESTOPTION;
			if (not QuestIon_MasterOptionPlayer["show_completeTag"]) then
				info.checked = 1;
			end
			info.func = QuestIon_FrameTitle_DropDown_CompleteTag;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_BACKGROUNDCOLOR;
			info.hasColorSwatch = 1;
			info.r = QuestIon_MasterOptionPlayer["red_frame_backdrop"];
			info.g = QuestIon_MasterOptionPlayer["green_frame_backdrop"];
			info.b = QuestIon_MasterOptionPlayer["blue_frame_backdrop"];
			info.opacity = 1 - QuestIon_MasterOptionPlayer["alpha_frame_backdrop"];
			info.swatchFunc = QuestIon_ColorPicker_Frame_Backdrop;
			info.func = UIDropDownMenuButton_OpenColorPicker;
			info.hasOpacity = 1;
			info.opacityFunc = QuestIon_Opacity_Frame_Backdrop;
			info.cancelFunc = QuestIon_ColorPickerCancel_Frame_Backdrop;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_BORDERCOLOR;
			info.hasColorSwatch = 1;
			info.r = QuestIon_MasterOptionPlayer["red_frame_border"];
			info.g = QuestIon_MasterOptionPlayer["green_frame_border"];
			info.b = QuestIon_MasterOptionPlayer["blue_frame_border"];
			info.opacity = 1 - QuestIon_MasterOptionPlayer["alpha_frame_border"];
			info.swatchFunc = QuestIon_ColorPicker_Frame_Border;
			info.func = UIDropDownMenuButton_OpenColorPicker;
			info.hasOpacity = 1;
			info.opacityFunc = QuestIon_Opacity_Frame_Border;
			info.cancelFunc = QuestIon_ColorPickerCancel_Frame_Border;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_RESETCOLORS;
			info.func = QuestIon_FrameTitle_DropDown_Default_Colors;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_FONTS;
			info.notClickable = 1;
			info.isTitle = 1;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_QUESTTITLESIZE;
			info.hasArrow = 1;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_QUESTOBJSIZE;
			info.hasArrow = 1;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_MOREFONTSIZES;
			if (QuestIon_MasterOptionPlayer["additional_font_sizes"]) then
				info.checked = 1;
			end
			info.func = QuestIon_FrameTitle_DropDown_Additional_Font_Sizes;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_EXITMENU;
			info.func = QuestIon_DropDown_ExitMenu;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

		elseif (UIDROPDOWNMENU_MENU_VALUE == QUESTIONTEXT_FADINGBEHAVIOR) then
			local info = {};
			info.text = QUESTIONTEXT_AUTOFADE;
			info.notClickable = 1;
			info.isTitle = 1;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_ENABLEAUTOFADE;
			if (QuestIon_MasterOptionPlayer["autofade_enabled"]) then
				info.checked = 1;
			end
			info.func = QuestIon_FrameTitle_DropDown_FadeEnable;
			info.tooltipTitle = QUESTIONTEXT_TT_AUTOFADE;
			info.tooltipText = QUESTIONTEXT_TT_AUTOFADEDESC;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_FADEINALPHA .. " |cff00c0ff" .. QuestIon_MasterOptionPlayer["autofade_inalpha"] .. "|r " .. QUESTIONTEXT_CLICKTOCHANGE;
			info.func = QuestIon_FrameTitle_DropDown_FadeInAlpha;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_FADEINTIME .. " |cff00c0ff" .. QuestIon_MasterOptionPlayer["autofade_intime"] .. "|r " .. QUESTIONTEXT_CLICKTOCHANGE;
			info.func = QuestIon_FrameTitle_DropDown_FadeInTime;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_FADEOUTALPHA .. " |cff00c0ff" .. QuestIon_MasterOptionPlayer["autofade_outalpha"] .. "|r " .. QUESTIONTEXT_CLICKTOCHANGE;
			info.func = QuestIon_FrameTitle_DropDown_FadeOutAlpha;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_FADEOUTTIME .. " |cff00c0ff" .. QuestIon_MasterOptionPlayer["autofade_outtime"] .. "|r " .. QUESTIONTEXT_CLICKTOCHANGE;
			info.func = QuestIon_FrameTitle_DropDown_FadeOutTime;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_AUTOMINIMIZE;
			info.notClickable = 1;
			info.isTitle = 1;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_ENABLEAUTOMIN;
			if (QuestIon_MasterOptionPlayer["autominimize_enabled"]) then
				info.checked = 1;
			end
			info.func = QuestIon_FrameTitle_DropDown_MinimizeEnable;
			info.tooltipTitle = QUESTIONTEXT_TT_AUTOMIN;
			info.tooltipText = QUESTIONTEXT_TT_AUTOMINDESC;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_SETNOHOVERKEY;
			if (QuestIon_MasterOptionPlayer["hover_restore_key"] == false) then
				info.checked = 1;
			end
			info.func = QuestIon_FrameTitle_DropDown_SetNoHoverKey;
			info.tooltipTitle = QUESTIONTEXT_TT_HOVERKEYS;
			info.tooltipText = QUESTIONTEXT_TT_HOVERKEYSDESC;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_SETALTHOVERKEY;
			if (QuestIon_MasterOptionPlayer["hover_restore_key"] == "alt") then
				info.checked = 1;
			end
			info.func = QuestIon_FrameTitle_DropDown_SetAltHoverKey;
			info.tooltipTitle = QUESTIONTEXT_TT_HOVERKEYS;
			info.tooltipText = QUESTIONTEXT_TT_HOVERKEYSDESC;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_SETCTRLHOVERKEY;
			if (QuestIon_MasterOptionPlayer["hover_restore_key"] == "ctrl") then
				info.checked = 1;
			end
			info.func = QuestIon_FrameTitle_DropDown_SetCtrlHoverKey;
			info.tooltipTitle = QUESTIONTEXT_TT_HOVERKEYS;
			info.tooltipText = QUESTIONTEXT_TT_HOVERKEYSDESC;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_SETSHIFTHOVERKEY;
			if (QuestIon_MasterOptionPlayer["hover_restore_key"] == "shift") then
				info.checked = 1;
			end
			info.func = QuestIon_FrameTitle_DropDown_SetShiftHoverKey;
			info.tooltipTitle = QUESTIONTEXT_TT_HOVERKEYS;
			info.tooltipText = QUESTIONTEXT_TT_HOVERKEYSDESC;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_EXITMENU;
			info.func = QuestIon_DropDown_ExitMenu;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

		elseif (UIDROPDOWNMENU_MENU_VALUE == QUESTIONTEXT_SHOWRESIZING) then
			info = {};
			info.text = QUESTIONTEXT_AUTOSHOW;
			info.notClickable = 1;
			info.isTitle = 1;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_UNMINOBJPICKUP;
			if (QuestIon_MasterOptionPlayer["unminimizeOnPickup"]) then
				info.checked = 1;
			end
			info.func = QuestIon_FrameTitle_DropDown_unminimizeOnPickup;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_RESIZECONTROL;
			info.notClickable = 1;
			info.isTitle = 1;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_RESIZEUPPERLEFT;
			if (QuestIon_MasterOptionPlayer["corner_mode"] == 1) then
				info.checked = 1;
			end
			info.func = QuestIon_FrameTitle_DropDown_UpperLeftMode;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_RESIZEUPPERRIGHT;
			if (QuestIon_MasterOptionPlayer["corner_mode"] == 2) then
				info.checked = 1;
			end
			info.func = QuestIon_FrameTitle_DropDown_UpperRightMode;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_RESIZELOWERLEFT;
			if (QuestIon_MasterOptionPlayer["corner_mode"] == 3) then
				info.checked = 1;
			end
			info.func = QuestIon_FrameTitle_DropDown_LowerLeftMode;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_RESIZELOWERRIGHT;
			if (QuestIon_MasterOptionPlayer["corner_mode"] == 4) then
				info.checked = 1;
			end
			info.func = QuestIon_FrameTitle_DropDown_LowerRightMode;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_EXITMENU;
			info.func = QuestIon_DropDown_ExitMenu;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);			

		elseif (UIDROPDOWNMENU_MENU_VALUE == QUESTIONTEXT_MISCOPTIONS) then
			local info = {};
			info.text = QUESTIONTEXT_MISC;
			info.notClickable = 1;
			info.isTitle = 1;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_SHOWQUESTTOOLTIPS;
			if (QuestIon_MasterOptionPlayer["frame_quest_tooltips"]) then
				info.checked = 1;
			end
			info.func = QuestIon_FrameTitle_DropDown_Quest_Tooltips;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_SHOWLOCATIONHEADERS;
			if (QuestIon_MasterOptionPlayer["showLocationHeaders"]) then
				info.checked = 1;
			end
			info.func = QuestIon_FrameTitle_DropDown_showLocationHeaders;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_SHOWDESCRIPTION;
			if (not QuestIon_MasterOptionPlayer["hideQuestDescriptions"]) then
				info.checked = 1;
			end
			info.func = QuestIon_FrameTitle_DropDown_hideQuestDescriptions;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_HIDEBUTTONS;
			if (QuestIon_MasterOptionPlayer["hide_buttons"]) then
				info.checked = 1;
			end
			info.func = QuestIon_FrameTitle_DropDown_ToggleButtons;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_PLAYSOUND;
			if (QuestIon_MasterOptionPlayer["playSoundQuestComplete"]) then
				info.checked = 1;
			end
			info.func = QuestIon_FrameTitle_DropDown_playSoundQuestComplete;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_TESTSOUND;
			info.func = QuestIon_FrameTitle_DropDown_testSound;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			local info = {};
			info.text = QUESTIONTEXT_QUESTFADE;
			info.notClickable = 1;
			info.isTitle = 1;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_DISABLEQUESTFADING;
			info.func = QuestIon_FrameTitle_DropDown_QuestFading;
			if (not QuestIon_MasterOptionPlayer["quest_fading_enabled"]) then
				info.checked = 1;
			end
			info.tooltipTitle = QUESTIONTEXT_DISABLEQUESTFADING;
			info.tooltipText = QUESTIONTEXT_TT_DISABLEQUESTFADINGDESC;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			local info = {};
			info.text = QUESTIONTEXT_UTILITYCOMMANDS;
			info.notClickable = 1;
			info.isTitle = 1;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_REFRESHWINDOW;
			info.func = QuestIon_WindowUpdate;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = "  " .. QUESTIONTEXT_RESETINTERNALQUESTLIST;
			info.func = QuestIon_FrameTitle_DropDown_Reset_Internal_Quests;
			info.notCheckable = 1;
			info.keepShownOnClick = 1;
			UIDropDownMenu_AddButton(info,2);

			info = {};
			info.text = QUESTIONTEXT_EXITMENU;
			info.func = QuestIon_DropDown_ExitMenu;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,2);			
		end
	elseif (level == 3) then
		if (UIDROPDOWNMENU_MENU_VALUE == "  " .. QUESTIONTEXT_QUESTTITLESIZE) then
			for index, value in CHAT_FONT_HEIGHTS do
				info = {};
				info.text = format(FONT_SIZE_TEMPLATE, value);
				info.value = value;
				info.func = QuestIon_FrameTitle_DropDown_FontSize_Title;

				if ( value == QuestIon_MasterOptionInfo["font_title_size"]) then
					info.checked = 1;
				end
				UIDropDownMenu_AddButton(info, 3);
			end
		elseif (UIDROPDOWNMENU_MENU_VALUE == "  " .. QUESTIONTEXT_QUESTOBJSIZE) then
			for index, value in CHAT_FONT_HEIGHTS do
				info = {};
				info.text = format(FONT_SIZE_TEMPLATE, value);
				info.value = value;
				info.func = QuestIon_FrameTitle_DropDown_FontSize_Objective;

				if ( value == QuestIon_MasterOptionInfo["font_objective_size"]) then
					info.checked = 1;
				end
				UIDropDownMenu_AddButton(info, 3);
			end

		elseif (UIDROPDOWNMENU_MENU_VALUE == QUESTIONTEXT_MOREQUESTS) then
			-- Get how many entries (quests + headers) and quests are in the Quest Log
			local numEntries, numQuests = GetNumQuestLogEntries()
			local currentQuestLocation = QuestIon_secondDropdownMenuLocation;

			-- Go through the entire list in the Quest Log
			for entryIndex = UIDROPDOWNMENU_MAXBUTTONS - 1, numEntries, 1 do
				-- Gather some information on our current Quest Log Entry
				local questLogTitleText, level, questTag, isHeader, isCollapsed, isComplete = GetQuestLogTitle(entryIndex);

				-- Is the current entry a Header?
				if (not isHeader) then
					SelectQuestLogEntry(entryIndex);
					
					-- Is the quest complete?
					if ( isComplete ) then
						-- If so, set the questTag
						questTag = COMPLETE;
					end

					-- Is the quest failed?
					if ( IsCurrentQuestFailed() ) then
						-- If so, set the questTag
						questTag = FAILED;
					end

					info = {};
					-- Get the Quest Title color using Blizzard's function from QuestLogFrame.lua
					local levelColor = GetDifficultyColor(level);

					-- Convert the levelColor to the |caarrggbb format
					local formattedLevelColor = string.format("|cff%02x%02x%02x", levelColor.r * 255, levelColor.g * 255, levelColor.b * 255);
					info.text = formattedLevelColor .. "  [" .. level .. "] " .. questLogTitleText .. "|r";
					info.value = questLogTitleText;
					if (QuestIon_MasterQuestPlayer[questLogTitleText]["isShown"]) then
						info.checked = 1;
					end
					info.func = QuestIon_FrameTitle_DropDown_ToggleShowQuest;
					info.keepShownOnClick = 1;

					-- Is there a questTag? (Complete, Failed, or Elite)
					if (questTag ~= nil) then
						info.tooltipTitle = formattedLevelColor .. questLogTitleText .. "|r (" .. questTag .. ")";
					else
						info.tooltipTitle = formattedLevelColor .. questLogTitleText .. "|r";
					end
					local questDescription, questObjectives = GetQuestLogQuestText();
					info.tooltipText = "|cffffffff" .. currentQuestLocation .. "\n|r|cffcccccc -" .. questObjectives .. "|r";

					UIDropDownMenu_AddButton(info,3);	
				else
					currentQuestLocation = questLogTitleText;
					QuestIon_secondDropdownMenuLocation = questLogTitleText;
					info = {};
					info.text = questLogTitleText;
					if (QuestIon_MasterQuestPlayer[questLogTitleText]["headerIsChecked"]) then
						info.checked = 1;
					end
					info.value = questLogTitleText;
					info.func = QuestIon_FrameTitle_DropDown_ToggleHeader;
					info.keepShownOnClick = 1;
					UIDropDownMenu_AddButton(info,3);
				end
			end
			info = {};
			info.text = QUESTIONTEXT_EXITMENU;
			info.func = QuestIon_DropDown_ExitMenu;
			info.notCheckable = 1;
			UIDropDownMenu_AddButton(info,3);
		end
	end
end

function QuestIon_DropDown_ToggleLock()
	if (QuestIon_MasterOptionPlayer["windowLocked"]) then
		QuestIon_MasterOptionPlayer["windowLocked"] = false;
	else
		QuestIon_MasterOptionPlayer["windowLocked"] = true;
	end
end

function QuestIon_FrameTitle_DropDown_default_isShown()
	if (QuestIon_MasterOptionPlayer["default_isShown"]) then
		QuestIon_MasterOptionPlayer["default_isShown"] = false;
	else
		QuestIon_MasterOptionPlayer["default_isShown"] = true;
	end
end

function QuestIon_FrameTitle_DropDown_default_isShownComplete()
	if (QuestIon_MasterOptionPlayer["default_isShownComplete"]) then
		QuestIon_MasterOptionPlayer["default_isShownComplete"] = false;
	else
		QuestIon_MasterOptionPlayer["default_isShownComplete"] = true;
	end
end

function QuestIon_FrameTitle_DropDown_default_showObjectives()
	if (QuestIon_MasterOptionPlayer["default_showObjectives"]) then
		QuestIon_MasterOptionPlayer["default_showObjectives"] = false;
	else
		QuestIon_MasterOptionPlayer["default_showObjectives"] = true;
	end
end

function QuestIon_FrameTitle_DropDown_default_showCompleteObjectives()
	if (QuestIon_MasterOptionPlayer["default_showCompleteObjectives"]) then
		QuestIon_MasterOptionPlayer["default_showCompleteObjectives"] = false;
	else
		QuestIon_MasterOptionPlayer["default_showCompleteObjectives"] = true;
	end
end

function QuestIon_FrameTitle_DropDown_default_showOnObjectivePickup()
	if (QuestIon_MasterOptionPlayer["default_showOnObjectivePickup"]) then
		QuestIon_MasterOptionPlayer["default_showOnObjectivePickup"] = false;
	else
		QuestIon_MasterOptionPlayer["default_showOnObjectivePickup"] = true;
	end
end

function QuestIon_FrameTitle_DropDown_default_locationInTitle()
	if (QuestIon_MasterOptionPlayer["default_locationInTitle"]) then
		QuestIon_MasterOptionPlayer["default_locationInTitle"] = false;
	else
		QuestIon_MasterOptionPlayer["default_locationInTitle"] = true;
	end
end

function QuestIon_FrameTitle_DropDown_default_locationInObjectives()
	if (QuestIon_MasterOptionPlayer["default_locationInObjectives"]) then
		QuestIon_MasterOptionPlayer["default_locationInObjectives"] = false;
	else
		QuestIon_MasterOptionPlayer["default_locationInObjectives"] = true;
	end
end

function QuestIon_FrameTitle_DropDown_default_showPartyInfo()
	if (QuestIon_MasterOptionPlayer["default_showPartyInfo"]) then
		QuestIon_MasterOptionPlayer["default_showPartyInfo"] = false;
	else
		QuestIon_MasterOptionPlayer["default_showPartyInfo"] = true;
	end
end

function QuestIon_FrameTitle_DropDown_default_showNotOnQuest()
	if (QuestIon_MasterOptionPlayer["default_showNotOnQuest"]) then
		QuestIon_MasterOptionPlayer["default_showNotOnQuest"] = false;
	else
		QuestIon_MasterOptionPlayer["default_showNotOnQuest"] = true;
	end
end

function QuestIon_FrameTitle_DropDown_default_sortMode()
	QuestIon_MasterOptionPlayer["default_sortMode"] = this.value;

	for i=1, UIDROPDOWNMENU_MAXBUTTONS do
		button = getglobal("DropDownList"..UIDROPDOWNMENU_MENU_LEVEL.."Button"..i);
		checkImage = getglobal("DropDownList"..UIDROPDOWNMENU_MENU_LEVEL.."Button"..i.."Check");
		if (button.value ~= nil) then
			if (button.value == this.value) then
				checkImage:Show();
				button.checked = nil;
			else
				checkImage:Hide();
				button.checked = 1;
			end
		end
	end
	QuestIon_WindowUpdate();
end

function QuestIon_FrameTitle_DropDown_default_filterByLocation()
	if (QuestIon_MasterOptionPlayer["default_filterByLocation"]) then
		QuestIon_MasterOptionPlayer["default_filterByLocation"] = false;
		QuestIon_FrameTitle_DropDown_ShowAllQuests();
	else
		QuestIon_MasterOptionPlayer["default_filterByLocation"] = true;
	end
	QuestIon_WindowUpdate();
end

function QuestIon_FrameTitle_DropDown_default_filterByLevel()
	if (QuestIon_MasterOptionPlayer["default_filterByLevel"]) then
		QuestIon_MasterOptionPlayer["default_filterByLevel"] = false;
		QuestIon_FrameTitle_DropDown_ShowAllQuests();
	else
		QuestIon_MasterOptionPlayer["default_filterByLevel"] = true;
	end
	QuestIon_WindowUpdate();
end

function QuestIon_FrameTitle_DropDown_default_filterRange()
	StaticPopup_Show("QuestIon_LevelRange");
	QuestIon_DropDown_ExitMenu();
end

function QuestIon_FrameTitle_DropDown_SetAllQuestsToDefault()
	for key in QuestIon_MasterQuestPlayer do
		if (QuestIon_MasterQuestPlayer[key]["isShown"] ~= nil) then
			QuestIon_MasterQuestPlayer[key]["isShown"] = QuestIon_MasterOptionPlayer["default_isShown"];
		end
		if (QuestIon_MasterQuestPlayer[key]["isShownComplete"] ~= nil) then
			QuestIon_MasterQuestPlayer[key]["isShownComplete"] = QuestIon_MasterOptionPlayer["default_isShownComplete"];
		end
		if (QuestIon_MasterQuestPlayer[key]["showObjectives"] ~= nil) then
			QuestIon_MasterQuestPlayer[key]["showObjectives"] = QuestIon_MasterOptionPlayer["default_showObjectives"];
		end
		if (QuestIon_MasterQuestPlayer[key]["showCompleteObjectives"] ~= nil) then
			QuestIon_MasterQuestPlayer[key]["showCompleteObjectives"] = QuestIon_MasterOptionPlayer["default_showCompleteObjectives"];
		end
		if (QuestIon_MasterQuestPlayer[key]["locationInTitle"] ~= nil) then
			QuestIon_MasterQuestPlayer[key]["locationInTitle"] = QuestIon_MasterOptionPlayer["default_locationInTitle"];
		end
		if (QuestIon_MasterQuestPlayer[key]["locationInObjectives"] ~= nil) then
			QuestIon_MasterQuestPlayer[key]["locationInObjectives"] = QuestIon_MasterOptionPlayer["default_locationInObjectives"];
		end
		if (QuestIon_MasterQuestPlayer[key]["showOnObjectivePickup"] ~= nil) then
			QuestIon_MasterQuestPlayer[key]["showOnObjectivePickup"] = QuestIon_MasterOptionPlayer["default_showOnObjectivePickup"];
		end
		if (QuestIon_MasterQuestPlayer[key]["showPartyInfo"] ~= nil) then
			QuestIon_MasterQuestPlayer[key]["showPartyInfo"] = QuestIon_MasterOptionPlayer["default_showPartyInfo"];
		end
		if (QuestIon_MasterQuestPlayer[key]["showNotOnQuest"] ~= nil) then
			QuestIon_MasterQuestPlayer[key]["showNotOnQuest"] = QuestIon_MasterOptionPlayer["default_showNotOnQuest"];
		end
	end
	QuestIon_WindowUpdate();
	this.checked = 1;
end

function QuestIon_FrameTitle_DropDown_ToggleShowQuest()
	if (QuestIon_MasterQuestPlayer[this.value]["isShown"]) then
		QuestIon_MasterQuestPlayer[this.value]["isShown"] = false;
	else
		QuestIon_MasterQuestPlayer[this.value]["isShown"] = true;
		if (QuestIon_MasterQuestPlayer[this.value]["isComplete"]) then
			QuestIon_MasterQuestPlayer[this.value]["isShownComplete"] = true;
		end
	end

	local currentLocation = "";
	local numEntries, numQuests = GetNumQuestLogEntries();
	
	-- Start with the item after the location header
	for entryIndex = 1, numEntries, 1 do
		local questLogTitleText, level, questTag, isHeader, isCollapsed, isComplete = GetQuestLogTitle(entryIndex);
		if (isHeader) then
			currentLocation = questLogTitleText;
		else
			if (questLogTitleText == this.value) then
				break;
			end
		end
	end

	QuestIon_MasterQuestPlayer[currentLocation]["headerIsChecked"] = false;

	for i=1, UIDROPDOWNMENU_MAXBUTTONS do
		button = getglobal("DropDownList"..UIDROPDOWNMENU_MENU_LEVEL.."Button"..i);
		checkImage = getglobal("DropDownList"..UIDROPDOWNMENU_MENU_LEVEL.."Button"..i.."Check");
		if (button.value ~= nil and QuestIon_MasterQuestPlayer[button.value] ~= nil and QuestIon_MasterQuestPlayer[button.value]["headerIsChecked"] ~= nil) then
			if (not QuestIon_MasterQuestPlayer[button.value]["headerIsChecked"]) then
				button:UnlockHighlight();
				checkImage:Hide();
				button.checked = nil;
			end
		end
	end

	QuestIon_WindowUpdate();
end

function QuestIon_FrameTitle_DropDown_ToggleHeader()
	if (QuestIon_MasterQuestPlayer[this.value]["headerIsChecked"]) then
		QuestIon_MasterQuestPlayer[this.value]["headerIsChecked"] = false;
	else
		QuestIon_MasterQuestPlayer[this.value]["headerIsChecked"] = true;
	end

	local foundHeader = false;
	local numEntries, numQuests = GetNumQuestLogEntries();
	
	-- Start with the item after the location header
	for entryIndex = 1, numEntries, 1 do
		local questLogTitleText, level, questTag, isHeader, isCollapsed, isComplete = GetQuestLogTitle(entryIndex);
		if (isHeader) then
			if (foundHeader) then
				break;
			end
			if (questLogTitleText == this.value) then
				foundHeader = true;
			end
		elseif (foundHeader) then
			QuestIon_MasterQuestPlayer[questLogTitleText]["isShown"] = QuestIon_MasterQuestPlayer[this.value]["headerIsChecked"];
			if (QuestIon_MasterQuestPlayer[this.value]["headerIsChecked"]) then
				if (QuestIon_MasterQuestPlayer[this.value]["isComplete"]) then
					QuestIon_MasterQuestPlayer[questLogTitleText]["isShownComplete"] = true;
				end
			end
		end
	end

	for i=1, UIDROPDOWNMENU_MAXBUTTONS do
		button = getglobal("DropDownList"..UIDROPDOWNMENU_MENU_LEVEL.."Button"..i);
		checkImage = getglobal("DropDownList"..UIDROPDOWNMENU_MENU_LEVEL.."Button"..i.."Check");
		if (button.value ~= nil and QuestIon_MasterQuestPlayer[button.value] ~= nil and QuestIon_MasterQuestPlayer[button.value]["isShown"] ~= nil) then
			if (QuestIon_MasterQuestPlayer[button.value]["isShown"]) then
				--button:LockHighlight();
				checkImage:Show();
				button.checked = 1;
			else
				--button:UnlockHighlight();
				checkImage:Hide();
				button.checked = nil;
			end
		end
	end

	QuestIon_WindowUpdate();
end

function QuestIon_FrameTitle_DropDown_ShowAllQuests()
	for key in QuestIon_MasterQuestPlayer do
		if (QuestIon_MasterQuestPlayer[key]["headerIsChecked"] == nil) then
			QuestIon_MasterQuestPlayer[key]["isShown"] = true;
			if (QuestIon_MasterQuestPlayer[key]["isComplete"]) then
				QuestIon_MasterQuestPlayer[key]["isShownComplete"] = true;
			end
		end
	end
	ExpandQuestHeader(0);
	QuestIon_WindowUpdate();
	--Fix due to how Blizzard handles keepShownOnClick menus
	this.checked = 1;
end

function QuestIon_FrameTitle_DropDown_HideAllQuests()
	for key in QuestIon_MasterQuestPlayer do
		if (QuestIon_MasterQuestPlayer[key]["headerIsChecked"] == nil) then
			QuestIon_MasterQuestPlayer[key]["isShown"] = false;
		end
	end
	QuestIon_WindowUpdate();
	this.checked = 1;
end

function QuestIon_FrameTitle_DropDown_SetObjectivePickup()
	for key in QuestIon_MasterQuestPlayer do
		if (QuestIon_MasterQuestPlayer[key]["headerIsChecked"] == nil) then
			QuestIon_MasterQuestPlayer[key]["showOnObjectivePickup"] = true;
		end
	end
	QuestIon_WindowUpdate();
	this.checked = 1;
end

function QuestIon_FrameTitle_DropDown_UnsetObjectivePickup()
	for key in QuestIon_MasterQuestPlayer do
		if (QuestIon_MasterQuestPlayer[key]["headerIsChecked"] == nil) then
			QuestIon_MasterQuestPlayer[key]["showOnObjectivePickup"] = false;
		end
	end
	QuestIon_WindowUpdate();
	this.checked = 1;
end

function QuestIon_FrameTitle_DropDown_ShowAllCompletedQuests()
	for key in QuestIon_MasterQuestPlayer do
		if (QuestIon_MasterQuestPlayer[key]["headerIsChecked"] == nil) then
			QuestIon_MasterQuestPlayer[key]["isShownComplete"] = true;
			if (QuestIon_MasterQuestPlayer[key]["isComplete"]) then
				QuestIon_MasterQuestPlayer[key]["isShown"] = true;
			end
		end
	end
	QuestIon_WindowUpdate();
	this.checked = 1;
end

function QuestIon_FrameTitle_DropDown_HideAllCompletedQuests()
	for key in QuestIon_MasterQuestPlayer do
		if (QuestIon_MasterQuestPlayer[key]["headerIsChecked"] == nil) then
			QuestIon_MasterQuestPlayer[key]["isShownComplete"] = false;
		end
	end
	QuestIon_WindowUpdate();
	this.checked = 1;
end

function QuestIon_FrameTitle_DropDown_ShowAllObjectives()
	for key in QuestIon_MasterQuestPlayer do
		if (QuestIon_MasterQuestPlayer[key]["headerIsChecked"] == nil) then
			QuestIon_MasterQuestPlayer[key]["showObjectives"] = true;
			QuestIon_MasterQuestPlayer[key]["showCompleteObjectives"] = true;
		end
	end
	QuestIon_WindowUpdate();
	this.checked = 1;
end

function QuestIon_FrameTitle_DropDown_HideAllObjectives()
	for key in QuestIon_MasterQuestPlayer do
		if (QuestIon_MasterQuestPlayer[key]["headerIsChecked"] == nil) then
			QuestIon_MasterQuestPlayer[key]["showObjectives"] = false;
		end
	end
	QuestIon_WindowUpdate();
	this.checked = 1;
end

function QuestIon_FrameTitle_DropDown_ShowAllCompleteObjectives()
	for key in QuestIon_MasterQuestPlayer do
		if (QuestIon_MasterQuestPlayer[key]["headerIsChecked"] == nil) then
			QuestIon_MasterQuestPlayer[key]["showCompleteObjectives"] = true;
		end
	end
	QuestIon_WindowUpdate();
	this.checked = 1;
end

function QuestIon_FrameTitle_DropDown_HideAllCompleteObjectives()
	for key in QuestIon_MasterQuestPlayer do
		if (QuestIon_MasterQuestPlayer[key]["headerIsChecked"] == nil) then
			QuestIon_MasterQuestPlayer[key]["showCompleteObjectives"] = false;
		end
	end
	QuestIon_WindowUpdate();
	this.checked = 1;
end

function QuestIon_FrameTitle_DropDown_ShowLocationInTitles()
	for key in QuestIon_MasterQuestPlayer do
		if (QuestIon_MasterQuestPlayer[key]["headerIsChecked"] == nil) then
			QuestIon_MasterQuestPlayer[key]["locationInTitle"] = true;
		end
	end
	QuestIon_WindowUpdate();
	this.checked = 1;
end

function QuestIon_FrameTitle_DropDown_HideLocationInTitles()
	for key in QuestIon_MasterQuestPlayer do
		if (QuestIon_MasterQuestPlayer[key]["headerIsChecked"] == nil) then
			QuestIon_MasterQuestPlayer[key]["locationInTitle"] = false;
		end
	end
	QuestIon_WindowUpdate();
	this.checked = 1;
end

function QuestIon_FrameTitle_DropDown_ShowLocationInObjectives()
	for key in QuestIon_MasterQuestPlayer do
		if (QuestIon_MasterQuestPlayer[key]["headerIsChecked"] == nil) then
			QuestIon_MasterQuestPlayer[key]["locationInObjectives"] = true;
		end
	end
	QuestIon_WindowUpdate();
	this.checked = 1;
end

function QuestIon_FrameTitle_DropDown_HideLocationInObjectives()
	for key in QuestIon_MasterQuestPlayer do
		if (QuestIon_MasterQuestPlayer[key]["headerIsChecked"] == nil) then
			QuestIon_MasterQuestPlayer[key]["locationInObjectives"] = false;
		end
	end
	QuestIon_WindowUpdate();
	this.checked = 1;
end

function QuestIon_FrameTitle_DropDown_ShowPartyStatus()
	for key in QuestIon_MasterQuestPlayer do
		if (QuestIon_MasterQuestPlayer[key]["headerIsChecked"] == nil) then
			QuestIon_MasterQuestPlayer[key]["showPartyInfo"] = true;
		end
	end
	QuestIon_WindowUpdate();
	this.checked = 1;
end

function QuestIon_FrameTitle_DropDown_HidePartyStatus()
	for key in QuestIon_MasterQuestPlayer do
		if (QuestIon_MasterQuestPlayer[key]["headerIsChecked"] == nil) then
			QuestIon_MasterQuestPlayer[key]["showPartyInfo"] = false;
		end
	end
	QuestIon_WindowUpdate();
	this.checked = 1;
end

function QuestIon_FrameTitle_DropDown_ShowAllInPartyStatus()
	for key in QuestIon_MasterQuestPlayer do
		if (QuestIon_MasterQuestPlayer[key]["headerIsChecked"] == nil) then
			QuestIon_MasterQuestPlayer[key]["showNotOnQuest"] = true;
		end
	end
	QuestIon_WindowUpdate();
	this.checked = 1;
end

function QuestIon_FrameTitle_DropDown_HidePartyNoQuest()
	for key in QuestIon_MasterQuestPlayer do
		if (QuestIon_MasterQuestPlayer[key]["headerIsChecked"] == nil) then
			QuestIon_MasterQuestPlayer[key]["showNotOnQuest"] = false;
		end
	end
	QuestIon_WindowUpdate();
	this.checked = 1;
end

function QuestIon_ColorPicker_Frame_Backdrop()
	local red,green,blue = ColorPickerFrame:GetColorRGB();
	QuestIon_MasterOptionPlayer["red_frame_backdrop"] = red;
	QuestIon_MasterOptionPlayer["green_frame_backdrop"] = green;
	QuestIon_MasterOptionPlayer["blue_frame_backdrop"] = blue;
	QuestIon_Frame_Backdrop:SetBackdropColor(red,green,blue,QuestIon_MasterOptionPlayer["alpha_frame_backdrop"]);
end

function QuestIon_ColorPickerCancel_Frame_Backdrop(prev)
	if (prev ~= nil) then
		red = prev.r;
		green = prev.g
		blue = prev.b
		alpha = 1.0 - prev.opacity
		QuestIon_MasterOptionPlayer["red_frame_backdrop"] = red;
		QuestIon_MasterOptionPlayer["green_frame_backdrop"] = green;
		QuestIon_MasterOptionPlayer["blue_frame_backdrop"] = blue;
		QuestIon_MasterOptionPlayer["alpha_frame_backdrop"] = alpha;
		QuestIon_Frame_Backdrop:SetBackdropColor(red,green,blue,alpha);
	end
end

function QuestIon_Opacity_Frame_Backdrop()
	local alpha = 1.0 - OpacitySliderFrame:GetValue();
	QuestIon_MasterOptionPlayer["alpha_frame_backdrop"] = alpha;
	QuestIon_Frame_Backdrop:SetBackdropColor(QuestIon_MasterOptionPlayer["red_frame_backdrop"],QuestIon_MasterOptionPlayer["green_frame_backdrop"],QuestIon_MasterOptionPlayer["blue_frame_backdrop"],alpha);
end

function QuestIon_ColorPicker_Frame_Border()
	local red,green,blue = ColorPickerFrame:GetColorRGB();
	QuestIon_MasterOptionPlayer["red_frame_border"] = red;
	QuestIon_MasterOptionPlayer["green_frame_border"] = green;
	QuestIon_MasterOptionPlayer["blue_frame_border"] = blue;
	QuestIon_Frame_Backdrop:SetBackdropBorderColor(red,green,blue,QuestIon_MasterOptionPlayer["alpha_frame_border"]);
end

function QuestIon_ColorPickerCancel_Frame_Border(prev)
	if (prev ~= nil) then
		red = prev.r;
		green = prev.g
		blue = prev.b
		alpha = 1.0 - prev.opacity
		QuestIon_MasterOptionPlayer["red_frame_border"] = red;
		QuestIon_MasterOptionPlayer["green_frame_border"] = green;
		QuestIon_MasterOptionPlayer["blue_frame_border"] = blue;
		QuestIon_MasterOptionPlayer["alpha_frame_border"] = alpha;
		QuestIon_Frame_Backdrop:SetBackdropBorderColor(red,green,blue,alpha);
	end
end

function QuestIon_Opacity_Frame_Border()
	local alpha = 1.0 - OpacitySliderFrame:GetValue();
	QuestIon_MasterOptionPlayer["alpha_frame_border"] = alpha;
	QuestIon_Frame_Backdrop:SetBackdropBorderColor(QuestIon_MasterOptionPlayer["red_frame_border"],QuestIon_MasterOptionPlayer["green_frame_border"],QuestIon_MasterOptionPlayer["blue_frame_border"],alpha);
end

function QuestIon_ColorPicker_Start_Objective()
	local red,green,blue = ColorPickerFrame:GetColorRGB();
	red = math.floor(red * 255);
	green = math.floor(green * 255);
	blue = math.floor(blue * 255);
	QuestIon_MasterOptionPlayer["color_obj_start_red"] = red
	QuestIon_MasterOptionPlayer["color_obj_start_green"] = green
	QuestIon_MasterOptionPlayer["color_obj_start_blue"] = blue
	QuestIon_WindowUpdate();
end

function QuestIon_ColorPickerCancel_Start_Objective(prev)
	if (prev ~= nil) then
		red = prev.r;
		green = prev.g
		blue = prev.b
		red = math.floor(red * 255);
		green = math.floor(green * 255);
		blue = math.floor(blue * 255);
		QuestIon_MasterOptionPlayer["color_obj_start_red"] = red
		QuestIon_MasterOptionPlayer["color_obj_start_green"] = green
		QuestIon_MasterOptionPlayer["color_obj_start_blue"] = blue
		QuestIon_WindowUpdate();
	end
end

function QuestIon_ColorPicker_End_Objective()
	local red,green,blue = ColorPickerFrame:GetColorRGB();
	red = math.floor(red * 255);
	green = math.floor(green * 255);
	blue = math.floor(blue * 255);
	QuestIon_MasterOptionPlayer["color_obj_end_red"] = red
	QuestIon_MasterOptionPlayer["color_obj_end_green"] = green
	QuestIon_MasterOptionPlayer["color_obj_end_blue"] = blue
	QuestIon_WindowUpdate();
end

function QuestIon_ColorPickerCancel_End_Objective(prev)
	if (prev ~= nil) then
		red = prev.r;
		green = prev.g
		blue = prev.b
		red = math.floor(red * 255);
		green = math.floor(green * 255);
		blue = math.floor(blue * 255);
		QuestIon_MasterOptionPlayer["color_obj_end_red"] = red
		QuestIon_MasterOptionPlayer["color_obj_end_green"] = green
		QuestIon_MasterOptionPlayer["color_obj_end_blue"] = blue
		QuestIon_WindowUpdate();
	end
end

function QuestIon_ColorPicker_Complete_Quest()
	local red,green,blue = ColorPickerFrame:GetColorRGB();
	red = math.floor(red * 255);
	green = math.floor(green * 255);
	blue = math.floor(blue * 255);
	QuestIon_MasterOptionPlayer["color_complete_quest_red"] = red
	QuestIon_MasterOptionPlayer["color_complete_quest_green"] = green
	QuestIon_MasterOptionPlayer["color_complete_quest_blue"] = blue
	QuestIon_WindowUpdate();
end

function QuestIon_ColorPickerCancel_Complete_Quest(prev)
	if (prev ~= nil) then
		red = prev.r;
		green = prev.g
		blue = prev.b
		red = math.floor(red * 255);
		green = math.floor(green * 255);
		blue = math.floor(blue * 255);
		QuestIon_MasterOptionPlayer["color_complete_quest_red"] = red
		QuestIon_MasterOptionPlayer["color_complete_quest_green"] = green
		QuestIon_MasterOptionPlayer["color_complete_quest_blue"] = blue
		QuestIon_WindowUpdate();
	end
end

function QuestIon_FrameTitle_DropDown_CompleteTag()
	if (QuestIon_MasterOptionPlayer["show_completeTag"]) then
		QuestIon_MasterOptionPlayer["show_completeTag"] = false;
	else
		QuestIon_MasterOptionPlayer["show_completeTag"] = true;
	end
	QuestIon_WindowUpdate();
end

function QuestIon_FrameTitle_DropDown_Default_Colors()
	QuestIon_MasterOptionPlayer["color_obj_start_red"] = 153;
	QuestIon_MasterOptionPlayer["color_obj_start_green"] = 153;
	QuestIon_MasterOptionPlayer["color_obj_start_blue"] = 153;
	QuestIon_MasterOptionPlayer["color_obj_end_red"] = 0;
	QuestIon_MasterOptionPlayer["color_obj_end_green"] = 128;
	QuestIon_MasterOptionPlayer["color_obj_end_blue"] = 255;
	QuestIon_MasterOptionPlayer["color_complete_quest_red"] = 128
	QuestIon_MasterOptionPlayer["color_complete_quest_green"] = 96
	QuestIon_MasterOptionPlayer["color_complete_quest_blue"] = 192
	QuestIon_MasterOptionPlayer["red_frame_backdrop"] = 0;
	QuestIon_MasterOptionPlayer["green_frame_backdrop"] = 0;
	QuestIon_MasterOptionPlayer["blue_frame_backdrop"] = 0;
	QuestIon_MasterOptionPlayer["alpha_frame_backdrop"] = 0.7;
	QuestIon_MasterOptionPlayer["red_frame_border"] = 1;
	QuestIon_MasterOptionPlayer["green_frame_border"] = 1;
	QuestIon_MasterOptionPlayer["blue_frame_border"] = 1;
	QuestIon_MasterOptionPlayer["alpha_frame_border"] = 1;
	QuestIon_Frame_Backdrop:SetBackdropColor(QuestIon_MasterOptionPlayer["red_frame_backdrop"],QuestIon_MasterOptionPlayer["green_frame_backdrop"],QuestIon_MasterOptionPlayer["blue_frame_backdrop"],QuestIon_MasterOptionPlayer["alpha_frame_backdrop"]);
	QuestIon_Frame_Backdrop:SetBackdropBorderColor(QuestIon_MasterOptionPlayer["red_frame_border"],QuestIon_MasterOptionPlayer["green_frame_border"],QuestIon_MasterOptionPlayer["blue_frame_border"],QuestIon_MasterOptionPlayer["alpha_frame_border"]);
	QuestIon_WindowUpdate();
	QuestIon_DropDown_ExitMenu();	
end

function QuestIon_FrameTitle_DropDown_FontSize_Title()
	QuestIon_MasterOptionInfo["font_title_size"] = this.value;
	for cleanupIndex = 1, QuestIon_numButtons, 1 do
		local QuestIon_QuestText = getglobal("QuestIon_QuestTitle_" .. cleanupIndex .. "_Text");
		QuestIon_QuestText:SetTextHeight(this.value);
	end
	QuestIon_Frame:SetScale(2);
	QuestIon_Frame:SetScale(UIParent:GetScale());
	QuestIon_WindowUpdate();
	QuestIon_DropDown_ExitMenu();
end

function QuestIon_FrameTitle_DropDown_FontSize_Objective()
	QuestIon_MasterOptionInfo["font_objective_size"] = this.value;
	for cleanupIndex = 1, QuestIon_numButtons, 1 do
		local QuestIon_QuestObjectives = getglobal("QuestIon_QuestTitle_" .. cleanupIndex .. "_Objectives");
		QuestIon_QuestObjectives:SetTextHeight(this.value);
	end
	QuestIon_Frame:SetScale(2);
	QuestIon_Frame:SetScale(UIParent:GetScale());
	QuestIon_WindowUpdate();
	QuestIon_DropDown_ExitMenu();
end

function QuestIon_FrameTitle_DropDown_Additional_Font_Sizes()
	if (QuestIon_MasterOptionPlayer["additional_font_sizes"]) then
		QuestIon_MasterOptionPlayer["additional_font_sizes"] = false;
		CHAT_FONT_HEIGHTS = {
			[1] = 12,
			[2] = 14,
			[3] = 16,
			[4] = 18
		};
	else
		QuestIon_MasterOptionPlayer["additional_font_sizes"] = true;
		CHAT_FONT_HEIGHTS = {
			[1] = 8,
			[2] = 10,
			[3] = 11,
			[4] = 12,
			[5] = 13,
			[6] = 14,
			[7] = 15,
			[8] = 16,
			[9] = 18,
			[10] = 20
		};
	end
end

function QuestIon_FrameTitle_DropDown_FadeEnable()
	if (QuestIon_MasterOptionPlayer["autofade_enabled"]) then
		QuestIon_MasterOptionPlayer["autofade_enabled"] = false;
		QuestIon_Frame:SetAlpha(1);
	else
		QuestIon_MasterOptionPlayer["autofade_enabled"] = true;
	end
end

function QuestIon_FrameTitle_DropDown_FadeInAlpha()
	StaticPopup_Show("QuestIon_FadeInAlpha");
	QuestIon_DropDown_ExitMenu();
end

function QuestIon_FrameTitle_DropDown_FadeInTime()
	StaticPopup_Show("QuestIon_FadeInTime");
	QuestIon_DropDown_ExitMenu();
end

function QuestIon_FrameTitle_DropDown_FadeOutAlpha()
	StaticPopup_Show("QuestIon_FadeOutAlpha");
	QuestIon_DropDown_ExitMenu();
end

function QuestIon_FrameTitle_DropDown_FadeOutTime()
	StaticPopup_Show("QuestIon_FadeOutTime");
	QuestIon_DropDown_ExitMenu();
end

function QuestIon_FrameTitle_DropDown_MinimizeEnable()
	if (QuestIon_MasterOptionPlayer["autominimize_enabled"]) then
		QuestIon_MasterOptionPlayer["autominimize_enabled"] = false;
	else
		QuestIon_MasterOptionPlayer["autominimize_enabled"] = true;
	end
end

function QuestIon_FrameTitle_DropDown_SetNoHoverKey()
	QuestIon_MasterOptionPlayer["hover_restore_key"] = false;
	QuestIon_DropDown_ExitMenu();
end

function QuestIon_FrameTitle_DropDown_SetAltHoverKey()
	QuestIon_MasterOptionPlayer["hover_restore_key"] = "alt";
	QuestIon_DropDown_ExitMenu();
end

function QuestIon_FrameTitle_DropDown_SetCtrlHoverKey()
	QuestIon_MasterOptionPlayer["hover_restore_key"] = "ctrl";
	QuestIon_DropDown_ExitMenu();
end

function QuestIon_FrameTitle_DropDown_SetShiftHoverKey()
	QuestIon_MasterOptionPlayer["hover_restore_key"] = "shift";
	QuestIon_DropDown_ExitMenu();
end

function QuestIon_FrameTitle_DropDown_Quest_Tooltips()
	if (QuestIon_MasterOptionPlayer["frame_quest_tooltips"]) then
		QuestIon_MasterOptionPlayer["frame_quest_tooltips"] = false;
	else
		QuestIon_MasterOptionPlayer["frame_quest_tooltips"] = true;
	end
end

function QuestIon_FrameTitle_DropDown_Reset_Internal_Quests()
	QuestIon_MasterQuestInfo[GetCVar("realmName").."-"..UnitName("player")] = {};
	QuestIon_MasterQuestPlayer = QuestIon_Quest_GetPlayer();
	QuestIon_WindowUpdate();
	this.checked = 1;
end

function QuestIon_FrameTitle_DropDown_UpperLeftMode()
	QuestIon_MasterOptionPlayer["corner_mode"] = 1;
	QuestIon_MouseUp();
	QuestIon_DropDown_ExitMenu();
end

function QuestIon_FrameTitle_DropDown_UpperRightMode()
	QuestIon_MasterOptionPlayer["corner_mode"] = 2;
	QuestIon_MouseUp();
	QuestIon_DropDown_ExitMenu();
end

function QuestIon_FrameTitle_DropDown_LowerLeftMode()
	QuestIon_MasterOptionPlayer["corner_mode"] = 3;
	QuestIon_MouseUp();
	QuestIon_DropDown_ExitMenu();
end

function QuestIon_FrameTitle_DropDown_LowerRightMode()
	QuestIon_MasterOptionPlayer["corner_mode"] = 4;
	QuestIon_MouseUp();
	QuestIon_DropDown_ExitMenu();
end

function QuestIon_FrameTitle_DropDown_QuestFading()
	if (QuestIon_MasterOptionPlayer["quest_fading_enabled"]) then
		QuestIon_MasterOptionPlayer["quest_fading_enabled"] = false;
	else
		QuestIon_MasterOptionPlayer["quest_fading_enabled"] = true;
	end
	QUEST_FADING_ENABLE = QuestIon_MasterOptionPlayer["quest_fading_enabled"];
end

function QuestIon_FrameTitle_DropDown_ToggleButtons()
	if (QuestIon_MasterOptionPlayer["hide_buttons"]) then
		QuestIon_MasterOptionPlayer["hide_buttons"] = false;
		QuestIon_CloseButton:Show();
		QuestIon_MinimizeButton:Show();
	else
		QuestIon_MasterOptionPlayer["hide_buttons"] = true;
		QuestIon_CloseButton:Hide();
		QuestIon_MinimizeButton:Hide();
	end
	QuestIon_WindowUpdate();
end

function QuestIon_FrameTitle_DropDown_unminimizeOnPickup()
	if (QuestIon_MasterOptionPlayer["unminimizeOnPickup"]) then
		QuestIon_MasterOptionPlayer["unminimizeOnPickup"] = false;
	else
		QuestIon_MasterOptionPlayer["unminimizeOnPickup"] = true;
	end
end

function QuestIon_FrameTitle_DropDown_showLocationHeaders()
	if (QuestIon_MasterOptionPlayer["showLocationHeaders"]) then
		QuestIon_MasterOptionPlayer["showLocationHeaders"] = false;
	else
		QuestIon_MasterOptionPlayer["showLocationHeaders"] = true;
	end
	QuestIon_WindowUpdate();
end

function QuestIon_FrameTitle_DropDown_hideQuestDescriptions()
	if (QuestIon_MasterOptionPlayer["hideQuestDescriptions"]) then
		QuestIon_MasterOptionPlayer["hideQuestDescriptions"] = false;
	else
		QuestIon_MasterOptionPlayer["hideQuestDescriptions"] = true;
	end
	QuestIon_WindowUpdate();
end

function QuestIon_FrameTitle_DropDown_playSoundQuestComplete()
	if (QuestIon_MasterOptionPlayer["playSoundQuestComplete"]) then
		QuestIon_MasterOptionPlayer["playSoundQuestComplete"] = false;
	else
		QuestIon_MasterOptionPlayer["playSoundQuestComplete"] = true;
	end
end

function QuestIon_FrameTitle_DropDown_testSound()
	PlaySoundFile(QUESTIONTEXT_SOUND_QUESTCOMPLETE);
	this.checked = 1;
end
----------------------------
-- Various Support Functions
----------------------------

function QuestIon_DropDown_ExitMenu()
	CloseDropDownMenus();
	QuestIon_OnLeave();
end

function QuestIon_Quest_GetPlayer()
-- Determine which Server/Character we have and setup to use that data for Quests
	if (QuestIon_MasterQuestInfo[GetCVar("realmName").."-"..UnitName("player")] == nil) then
		QuestIon_MasterQuestInfo[GetCVar("realmName").."-"..UnitName("player")] = {};
	end
	return QuestIon_MasterQuestInfo[GetCVar("realmName").."-"..UnitName("player")];
end

function QuestIon_Option_GetPlayer()
-- Determine which Server/Character we have and setup to use that data for Options
	if (QuestIon_MasterOptionInfo[GetCVar("realmName").."-"..UnitName("player")] == nil) then
		QuestIon_MasterOptionInfo[GetCVar("realmName").."-"..UnitName("player")] = {};
	end
	return QuestIon_MasterOptionInfo[GetCVar("realmName").."-"..UnitName("player")];
end

function QuestIon_SetRestore()
	for Index = 1, QuestIon_numButtons, 1 do
		local QuestIon_QuestText = getglobal("QuestIon_QuestTitle_" .. Index);
		local QuestIon_QuestObjectives = getglobal("QuestIon_QuestTitle_" .. Index .. "_Objectives");
		local QuestIon_QuestTag = getglobal("QuestIon_QuestTitle_" .. Index .. "_Tag");
		QuestIon_QuestText:Show();
		QuestIon_QuestTag:Show();
		QuestIon_QuestObjectives:Show();
	end
	QuestIon_MasterOptionPlayer["frame_minimized"] = false;
	QuestIon_WindowUpdate();
end

function QuestIon_SetMinimized()
	for index = 1, QuestIon_numButtons, 1 do
		local QuestIon_QuestText = getglobal("QuestIon_QuestTitle_" .. index);
		local QuestIon_QuestObjectives = getglobal("QuestIon_QuestTitle_" .. index .. "_Objectives");
		local QuestIon_QuestTag = getglobal("QuestIon_QuestTitle_" .. index .. "_Tag");
		QuestIon_QuestText:Hide();
		QuestIon_QuestTag:Hide();
		QuestIon_QuestObjectives:Hide();
	end
	QuestIon_MasterOptionPlayer["frame_minimized"] = true;
	QuestIon_Frame:SetHeight(30);
	if (QuestIon_MasterOptionPlayer["hide_buttons"]) then
		QuestIon_Frame:SetWidth(120);
	else
		QuestIon_Frame:SetWidth(160);
	end
end

function QuestIon_ToggleMinimized()
	if (QuestIon_MasterOptionPlayer["frame_minimized"] == false) then
		--Minimize
		QuestIon_SetMinimized();
	else
		--Restore
		QuestIon_SetRestore();
	end
end

function QuestIon_ToggleVisible()
	if (QuestIon_Frame:IsVisible()) then
		QuestIon_SlashHandler("hide");
	else
		QuestIon_SlashHandler("show");
	end
end

function QuestIon_InternalQuestCleanup()
	local questsToProcess = {};
	for key in QuestIon_MasterQuestPlayer do
		questsToProcess[key] = true;
	end

	local numEntries, numQuests = GetNumQuestLogEntries()
	
	-- Go through the entire list in the Quest Log
	for entryIndex = 1, numEntries, 1 do
		-- Gather some information on our current Quest Log Entry
		local questLogTitleText, level, questTag, isHeader, isCollapsed, isComplete = GetQuestLogTitle(entryIndex);
		questsToProcess[questLogTitleText] = false;
	end

	for key in QuestIon_MasterQuestPlayer do
		if (questsToProcess[key]) then
			QuestIon_MasterQuestPlayer[key] = nil;
		end
	end
end

function QuestIon_ChatPrint(msg)
	DEFAULT_CHAT_FRAME:AddMessage(msg);
end

--------------------------------------------
-- Function that replace Blizzard's function
--------------------------------------------

-- This has been disabled in 1.6.0 since lots of addons replace this function now.

-- This is Blizzard's function as of 4/19/2005 (1300)
-- Specific changes are noted within the function

function ToggleDropDownMenuUpdate_ToggleDropDownMenu(level, value, dropDownFrame, anchorName, xOffset, yOffset)
	if ( not level ) then
		level = 1;
	end
	UIDROPDOWNMENU_MENU_LEVEL = level;
	UIDROPDOWNMENU_MENU_VALUE = value;
	local listFrame = getglobal("DropDownList"..level);
	local listFrameName = "DropDownList"..level;
	local tempFrame;
	local point, relativePoint, relativeTo;
	if ( not dropDownFrame ) then
		tempFrame = this:GetParent();
	else
		tempFrame = dropDownFrame;
	end
	if ( listFrame:IsVisible() and (UIDROPDOWNMENU_OPEN_MENU == tempFrame:GetName()) ) then
		listFrame:Hide();
	else
		-- Hide the listframe anyways since it is redrawn OnShow() 
		listFrame:Hide();
		
		-- Frame to anchor the dropdown menu to
		local anchorFrame;

		-- Display stuff
		-- Level specific stuff
		if ( level == 1 ) then
			if ( not dropDownFrame ) then
				dropDownFrame = this:GetParent();
			end
			UIDROPDOWNMENU_OPEN_MENU = dropDownFrame:GetName();
			listFrame:ClearAllPoints();
			-- If there's no specified anchorName then use left side of the dropdown menu
			if ( not anchorName ) then
				-- See if the anchor was set manually using setanchor
				if ( dropDownFrame.xOffset ) then
					xOffset = dropDownFrame.xOffset;
				end
				if ( dropDownFrame.yOffset ) then
					yOffset = dropDownFrame.yOffset;
				end
				if ( dropDownFrame.point ) then
					point = dropDownFrame.point;
				end
				if ( dropDownFrame.relativeTo ) then
					relativeTo = dropDownFrame.relativeTo;
				else
					relativeTo = UIDROPDOWNMENU_OPEN_MENU.."Left";
				end
				if ( dropDownFrame.relativePoint ) then
					relativePoint = dropDownFrame.relativePoint;
				end
			elseif ( anchorName == "cursor" ) then
				relativeTo = "UIParent";
				local cursorX, cursorY = GetCursorPosition();
				if ( not xOffset ) then
					xOffset = 0;
				end
				if ( not yOffset ) then
					yOffset = 0;
				end
				xOffset = cursorX + xOffset;
				yOffset = cursorY + yOffset;
------------------------------------------------------------
-- CHANGE: The anchorName must be set to the UIParent when setting based on the cursor
-- New code:
				anchorName = "UIParent";
-- :CHANGE
------------------------------------------------------------
			else
				relativeTo = anchorName;
			end
			if ( not xOffset or not yOffset ) then
				xOffset = 8;
				yOffset = 22;
			end
			if ( not point ) then
				point = "TOPLEFT";
			end
			if ( not relativePoint ) then
				relativePoint = "BOTTOMLEFT";
			end
			listFrame:SetPoint(point, relativeTo, relativePoint, xOffset, yOffset);
		else
			if ( not dropDownFrame ) then
				dropDownFrame = getglobal(UIDROPDOWNMENU_OPEN_MENU);
			end
			listFrame:ClearAllPoints();
			-- If this is a dropdown button, not the arrow anchor it to itself
			if ( strsub(this:GetParent():GetName(), 0,12) == "DropDownList" and strlen(this:GetParent():GetName()) == 13 ) then
				anchorFrame = this:GetName();
			else
				anchorFrame = this:GetParent():GetName();
			end
			listFrame:SetPoint("TOPLEFT", anchorFrame, "TOPRIGHT", 0, 0);
		end
		
		-- Change list box appearance depending on display mode
		if ( dropDownFrame and dropDownFrame.displayMode == "MENU" ) then
			getglobal(listFrameName.."Backdrop"):Hide();
			getglobal(listFrameName.."MenuBackdrop"):Show();
		else
			getglobal(listFrameName.."Backdrop"):Show();
			getglobal(listFrameName.."MenuBackdrop"):Hide();
		end

		UIDropDownMenu_Initialize(dropDownFrame, dropDownFrame.initialize, nil, level);
		-- If no items in the drop down don't show it
		if ( listFrame.numButtons == 0 ) then
			return;
		end

		-- Check to see if the dropdownlist is off the screen, if it is anchor it to the top of the dropdown button
		listFrame:Show();
		-- Hack since GetCenter() is returning coords relative to 1024x768
		local x, y = listFrame:GetCenter();
		-- Hack will fix this in next revision of dropdowns
		if ( not x or not y ) then
			listFrame:Hide();
			return;
		end
		-- Determine whether the menu is off the screen or not
		local offscreenY, offscreenX;
		if ( (y - listFrame:GetHeight()/2) < 0 ) then
			offscreenY = 1;
		end

------------------------------------------------------------
-- CHANGE: Since GetCenter returns coords relative to 1024x768, the right bound is 
-- always 1024 not what the current GetScreenWidth returns.
-- Original code: if ( x + listFrame:GetWidth()/2 > GetScreenWidth() ) then

-- New code:
		if ( x + listFrame:GetWidth()/2 > 1024 ) then

-- :CHANGE
------------------------------------------------------------
			offscreenX = 1;		
		end

		
		--  If level 1 can only go off the bottom of the screen
		if ( level == 1 ) then
------------------------------------------------------------
-- CHANGE: Make sure level 1 menus don't go off the screen to the right or bottom.
-- Original code:
-- if ( offscreenY ) then
--	listFrame:ClearAllPoints();
--	listFrame:SetPoint("BOTTOMLEFT", anchorName, "TOPLEFT", xOffset, -yOffset);
-- end

-- New code:
			-- Based on Blizzard's code for the other menu levels
			-- Now the first level of a drop down menu can't be off screen
			if ( offscreenY ) then
				if ( offscreenX ) then
					-- Off the bottom and right of the screen
					listFrame:ClearAllPoints();
					listFrame:SetPoint("BOTTOMRIGHT", anchorName, "BOTTOMLEFT", xOffset, yOffset);
				else
					-- Off the bottom of the screen
					listFrame:ClearAllPoints();
					listFrame:SetPoint("BOTTOMLEFT", anchorName, "BOTTOMLEFT", xOffset, yOffset);
				end
			else
				if ( offscreenX ) then
					-- Off the right of the screen
					listFrame:ClearAllPoints();
					listFrame:SetPoint("TOPRIGHT", anchorName, "BOTTOMLEFT", xOffset, yOffset);
				-- else
					-- Normal, so do nothing
				end
			end
-- :CHANGE
------------------------------------------------------------
		else
			local anchorPoint, relativePoint, offsetX, offsetY;
			if ( offscreenY ) then
				if ( offscreenX ) then
					anchorPoint = "BOTTOMRIGHT";
					relativePoint = "BOTTOMLEFT";
					offsetX = 0;
					offsetY = -14;
				else
					anchorPoint = "BOTTOMLEFT";
					relativePoint = "BOTTOMRIGHT";
					offsetX = 0;
					offsetY = -14;
				end
			else
				if ( offscreenX ) then
					anchorPoint = "TOPRIGHT";
					relativePoint = "TOPLEFT";
					offsetX = 0;
					offsetY = 14;
				else
					anchorPoint = "TOPLEFT";
					relativePoint = "TOPRIGHT";
					offsetX = 0;
					offsetY = 14;
				end
			end
			listFrame:ClearAllPoints();
			listFrame:SetPoint(anchorPoint, anchorFrame, relativePoint, offsetX, offsetY);
------------------------------------------------------------
-- CHANGE: In some cases a long menu that would have been off screen low was set to avoid that but would
-- go off screen high due to its height. I attempt to place the menu in the middle. Not a perfect solution,
-- but it should help in most cases. Also, I didn't want to change any default behaviors unless the drop 
-- down menu was off the screen.
-- New code:
			local x2, y2 = listFrame:GetCenter();
			local stilloffscreenY;
			-- 768 or GetScreenHeight?
			if ( (y2 + listFrame:GetHeight()/2) > 768 ) then
				if ( offscreenX ) then
					anchorPoint = "RIGHT";
					relativePoint = "BOTTOMLEFT";
					offsetX = 0;
					offsetY = -14;
				else
					anchorPoint = "LEFT";
					relativePoint = "BOTTOMRIGHT";
					offsetX = 0;
					offsetY = -14;
				end
				listFrame:ClearAllPoints();
				listFrame:SetPoint(anchorPoint, anchorFrame, relativePoint, offsetX, offsetY);
			end
-- :CHANGE
------------------------------------------------------------
		end
	end
end